searchData={"items":[{"type":"behaviour","title":"EventStore","doc":"EventStore allows you to define one or more event store modules to append,\nread, and subscribe to streams of events.\n\nIt uses PostgreSQL (v9.5 or later) as the underlying storage engine.","ref":"EventStore.html"},{"type":"behaviour","title":"Defining an event store - EventStore","doc":"An event store module is defined in your own application as follows:\n\n    defmodule MyApp.EventStore do\n      use EventStore, otp_app: :my_app\n\n      # Optional `init/1` function to modify config at runtime.\n      def init(config) do\n        {:ok, config}\n      end\n    end\n\nWhere the configuration for the event store must be in your application\nenvironment, usually defined in `config/config.exs`:\n\n    config :my_app, MyApp.EventStore,\n      serializer: EventStore.JsonSerializer,\n      username: \"postgres\",\n      password: \"postgres\",\n      database: \"eventstore\",\n      hostname: \"localhost\"\n\nOr use a URL to connect instead:\n\n    config :my_app, MyApp.EventStore,\n      serializer: EventStore.JsonSerializer,\n      url: \"postgres://postgres:postgres@localhost/eventstore\"\n\n**Note:** To use an EventStore with Commanded you should configure the event\nstore to use Commanded's JSON serializer which provides additional support for\nJSON decoding:\n\n    config :my_app, MyApp.EventStore,\n      serializer: Commanded.Serialization.JsonSerializer\n\nThe event store module defines a `start_link/1` function that needs to be\ninvoked before using the event store. In general, this function is not\ncalled directly, but included as part of your application supervision tree.\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file\ncontaining the application start callback that defines and starts your\nsupervisor. You just need to edit the `start/2` function to start the event\nstore in your application's supervisor:\n\n      def start(_type, _args) do\n        children = [\n          MyApp.EventStore\n        ]\n\n        opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n        Supervisor.start_link(children, opts)\n      end\n\nEach event store module (e.g. `MyApp.EventStore`) provides a public API to\nread events from and write events to an event stream, and subscribe to event\nnotifications.","ref":"EventStore.html#module-defining-an-event-store"},{"type":"behaviour","title":"Postgres schema - EventStore","doc":"By default the `public` schema will be used for event store tables. An event\nstore can be configured to use an alternate Postgres schema:\n\n    defmodule MyApp.EventStore do\n      use EventStore, otp_app: :my_app, schema: \"schema_name\"\n    end\n\nOr provide the schema as an option in the `init/1` callback function:\n\n    defmodule MyApp.EventStore do\n      use EventStore, otp_app: :my_app\n\n      def init(config) do\n        {:ok, Keyword.put(config, :schema, \"schema_name\")}\n      end\n    end\n\nOr define it in environment config when configuring the database connection\nsettings:\n\n    # config/config.exs\n    config :my_app, MyApp.EventStore, schema: \"schema_name\"\n\nThis feature allows you to define and start multiple event stores sharing a\nsingle Postgres database, but with their data isolated and segregated by\nschema.\n\nNote the `mix event_store. ` tasks to create, initialize, and drop an\nevent store database will also handle creating and/or dropping the schema.","ref":"EventStore.html#module-postgres-schema"},{"type":"behaviour","title":"Dynamic named event store - EventStore","doc":"An event store can be started multiple times by providing a name when\nstarting. The name must be provided as an option to all event store operations\nto identify the correct instance.\n\n#","ref":"EventStore.html#module-dynamic-named-event-store"},{"type":"behaviour","title":"Example - EventStore","doc":"Define an event store:\n\n    defmodule MyApp.EventStore do\n      use EventStore, otp_app: :my_app\n    end\n\nStart multiple instances of the event store, each with a unique name:\n\n    {:ok, _pid} = EventStore.start_link(name: :eventstore1)\n    {:ok, _pid} = EventStore.start_link(name: :eventstore2)\n    {:ok, _pid} = EventStore.start_link(name: :eventstore3)\n\nUse a dynamic event store by providing its name as an option to each function:\n\n    :ok = EventStore.append_to_stream(stream_uuid, expected_version, events, name: :eventstore1)\n\n    {:ok, events} = EventStore.read_stream_forward(stream_uuid, 0, 1_000, name: :eventstore1)","ref":"EventStore.html#module-example"},{"type":"behaviour","title":"Dynamic schemas - EventStore","doc":"This feature also allows you to start each event store instance using a\ndifferent schema:\n\n    {:ok, _pid} = EventStore.start_link(name: :tenant1, schema: \"tenant1\")\n    {:ok, _pid} = EventStore.start_link(name: :tenant2, schema: \"tenant2\")\n\nOr start supervised:\n\n    children =\n      for tenant <- [:tenant1, :tenant2, :tenant3] do\n        {MyApp.EventStore, name: tenant, schema: \"#{tenant}\"}\n      end\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n\n    Supervisor.start_link(children, opts)\n\nThe above can be used for multi-tenancy where the data for each tenant is\nstored in a separate, isolated schema.","ref":"EventStore.html#module-dynamic-schemas"},{"type":"behaviour","title":"Shared database connection pools - EventStore","doc":"By default each event store will start its own `Postgrex` database connection\npool. The size of the pool is configured with the `pool_size` config option.\n\nWhen you have multiple event stores running you will also end up with multiple\nconnection pools. If they are all connecting to the same physical Postgres\ndatabase then it can be useful to share a single pool amongst all event\nstores. Use the `shared_connection_pool` config option to specify a name for\nthe shared connection pool. Then configure the event stores you'd like to\nshare the pool with the same name.\n\nThis can be done in config:\n\n    # config/config.exs\n    config :my_app, MyApp.EventStore, shared_connection_pool: :shared_pool\n\nOr when starting the event stores, such as via a `Supervisor`:\n\n    Supervisor.start_link(\n      [\n        {MyApp.EventStore, name: :eventstore1, shared_connection_pool: :shared_pool},\n        {MyApp.EventStore, name: :eventstore2, shared_connection_pool: :shared_pool},\n        {MyApp.EventStore, name: :eventstore3, shared_connection_pool: :shared_pool}\n      ], opts)","ref":"EventStore.html#module-shared-database-connection-pools"},{"type":"behaviour","title":"Using an existing database connection or transaction - EventStore","doc":"In some situations you might want to execute the event store operations using\nan existing Postgres database connection or transaction. For instance, if you\nwant to persist changes to one or more other tables, such as a read-model\nprojection.\n\nTo do this you can provide a Postgrex connection process or transaction as a\n`:conn` option to any of the supported `EventStore` functions.\n\n    {:ok, pid} = Postgrex.start_link(config)\n\n    Postgrex.transaction(pid, fn conn ->\n      :ok = EventStore.append_to_stream(stream_uuid, expected_version, events, conn: conn)\n    end)\n\nThis can also be used with an Ecto `Repo` which is configured to use the\nPostgres SQL adapter. The connection process may be looked up as follows:\n\n    Repo.transaction(fn ->\n      %{pid: pool} = Ecto.Adapter.lookup_meta(Repo)\n\n      conn = Process.get({Ecto.Adapters.SQL, pool})\n\n      :ok = EventStore.append_to_stream(stream_uuid, expected_version, events, conn: conn)\n    end)\n\n---","ref":"EventStore.html#module-using-an-existing-database-connection-or-transaction"},{"type":"behaviour","title":"Guides - EventStore","doc":"Please refer to the following guides to learn more:\n\n- [Getting started](getting-started.html)\n- [Usage](usage.html)\n- [Subscriptions](subscriptions.html)\n- [Running on a cluster of nodes](cluster.html)\n- [Event serialization](event-serialization.html)\n- [Upgrading an existing EventStore database](upgrades.html)\n\n---","ref":"EventStore.html#module-guides"},{"type":"callback","title":"EventStore.ack/2","doc":"Acknowledge receipt of the given events received from a subscription.\n\nAccepts a single `EventStore.RecordedEvent` struct, a list of\n`EventStore.RecordedEvent`s, or the event number of the recorded event to\nacknowledge.","ref":"EventStore.html#c:ack/2"},{"type":"function","title":"EventStore.all_instances/0","doc":"Returns all running EventStore instances.\n\nNote that order is not guaranteed.","ref":"EventStore.html#all_instances/0"},{"type":"callback","title":"EventStore.append_to_stream/4","doc":"Append one or more events to a stream atomically.\n\n  - `stream_uuid` is used to uniquely identify a stream.\n\n  - `expected_version` is used for optimistic concurrency checks.\n    You can provide a non-negative integer to specify the expected stream\n    version. This is used to ensure you can only append to the stream if it is\n    at exactly that version.\n\n    You can also provide one of the following values to alter the concurrency\n    check behaviour:\n\n    - `:any_version` - No concurrency checking and allow any stream version\n      (including no stream).\n    - `:no_stream` - Ensure the stream does not exist.\n    - `:stream_exists` - Ensure the stream exists.\n\n  - `events` is a list of `%EventStore.EventData{}` structs.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database transaction, in\n      milliseconds. Defaults to 15,000ms.\n\nReturns `:ok` on success, or an `{:error, reason}` tagged tuple. The returned\nerror may be due to one of the following reasons:\n\n  * `{:error, :wrong_expected_version}` when the actual stream version differs\n    from the provided expected version.\n  * `{:error, :stream_exists}` when the stream exists, but expected version\n    was `:no_stream`.\n  * `{:error, :stream_not_found}` when the stream does not exist, but\n    expected version was `:stream_exists`.","ref":"EventStore.html#c:append_to_stream/4"},{"type":"callback","title":"EventStore.config/0","doc":"Returns the event store configuration stored in the `:otp_app` environment.","ref":"EventStore.html#c:config/0"},{"type":"callback","title":"EventStore.delete_all_streams_subscription/2","doc":"Delete an existing persistent subscription to all streams.\n\n  - `subscription_name` is used to identify the existing subscription to\n    remove.\n\nReturns `:ok` on success.","ref":"EventStore.html#c:delete_all_streams_subscription/2"},{"type":"callback","title":"EventStore.delete_snapshot/2","doc":"Delete a previously recorded snapshop for a given source.\n\nReturns `:ok` on success, or when the snapshot does not exist.","ref":"EventStore.html#c:delete_snapshot/2"},{"type":"callback","title":"EventStore.delete_stream/4","doc":"Delete an existing stream.\n\n  - `stream_uuid` identity of the stream to be deleted.\n\n  - `expected_version` is used for optimistic concurrency checking.\n    You can provide a non-negative integer to specify the expected stream\n    version. This is used to ensure you can only delete a stream if it is\n    at exactly that version.\n\n    You can also provide one of the following values to alter the concurrency\n    checking behaviour:\n\n    - `:any_version` - No concurrency check, allow any stream version.\n    - `:stream_exists` - Ensure the stream exists, at any version.\n\n  - `type` - used to indicate how the stream is deleted:\n\n    - `:soft` - the stream is marked as deleted, but no events are removed.\n    - `:hard` - the stream and its events are permanently deleted from the\n      database.\n\n    Soft deletion is the default if the type is not provided.\n\nReturns `:ok` on success or an error tagged tuple on failure.\n\n#","ref":"EventStore.html#c:delete_stream/4"},{"type":"callback","title":"Soft delete - EventStore.delete_stream/4","doc":"Will mark the stream as deleted, but will not delete its events. Events from\nsoft deleted streams will still appear in the globally ordered all events\n(`$all`) stream and in any linked streams.\n\nA soft deleted stream cannot be read nor appended to. Subscriptions to the\ndeleted stream will not receive any events but subscriptions containing linked\nevents from the deleted stream, such as the global all events stream, will\nstill receive events from the deleted stream.\n\n#","ref":"EventStore.html#c:delete_stream/4-soft-delete"},{"type":"callback","title":"Hard delete - EventStore.delete_stream/4","doc":"Will permanently delete the stream and its events. **This is irreversible and\nwill remove data**. Events will be removed from the globally ordered all\nevents stream and any linked streams.\n\nAfter being hard deleted, a stream can later be appended to and read as if it\nhad never existed.\n\n#","ref":"EventStore.html#c:delete_stream/4-hard-delete"},{"type":"callback","title":"Examples - EventStore.delete_stream/4","doc":"##","ref":"EventStore.html#c:delete_stream/4-examples"},{"type":"callback","title":"Soft delete a stream - EventStore.delete_stream/4","doc":"Delete a stream at any version:\n\n    :ok = MyApp.EventStore.delete_stream(\"stream1\", :any_version, :soft)\n\nDelete a stream at an expected version:\n\n    :ok = MyApp.EventStore.delete_stream(\"stream2\", 3, :soft)\n\nDelete stream will use soft delete by default so you can omit the type:\n\n    :ok = MyApp.EventStore.delete_stream(\"stream1\", :any_version)\n\n##","ref":"EventStore.html#c:delete_stream/4-soft-delete-a-stream"},{"type":"callback","title":"Hard delete a stream - EventStore.delete_stream/4","doc":"Since hard deletes are destructive and irreversible they are disabled by\ndefault. To use hard deletes you must first enable them for the event store:\n\n    defmodule MyApp.EventStore do\n      use EventStore, otp_app: :my_app, enable_hard_deletes: true\n    end\n\nOr via config:\n\n    # config/config.exs\n    config :my_app, MyApp.EventStore, enable_hard_deletes: true\n\nHard delete a stream at any version:\n\n    :ok = MyApp.EventStore.delete_stream(\"stream1\", :any_version, :hard)\n\nHard delete a stream that should exist:\n\n    :ok = MyApp.EventStore.delete_stream(\"stream2\", :stream_exists, :hard)","ref":"EventStore.html#c:delete_stream/4-hard-delete-a-stream"},{"type":"callback","title":"EventStore.delete_subscription/3","doc":"Delete an existing persistent subscription.\n\n  - `stream_uuid` is the stream the subscription is subscribed to.\n\n  - `subscription_name` is used to identify the existing subscription to\n    remove.\n\nReturns `:ok` on success.","ref":"EventStore.html#c:delete_subscription/3"},{"type":"callback","title":"EventStore.init/1","doc":"A callback executed when the event store starts or when configuration is read.\n\nIt must return `{:ok, keyword}` with the updated list of configuration.","ref":"EventStore.html#c:init/1"},{"type":"callback","title":"EventStore.link_to_stream/4","doc":"Link one or more existing events to another stream.\n\nAllows you to construct streams containing events already appended to any\nother stream. This is more efficient than copying events between streams since\nonly a reference to the existing event is created.\n\n  - `stream_uuid` is used to uniquely identify the target stream.\n\n  - `expected_version` is used for optimistic concurrency checks.\n    You can provide a non-negative integer to specify the expected stream\n    version. This is used to ensure you can only append to the stream if it is\n    at exactly that version.\n\n    You can also provide one of the following values to affect the concurrency\n    check behaviour:\n\n    - `:any_version` - No concurrency checking; allow any stream version\n      (including no stream).\n    - `:no_stream` - Ensure the stream does not exist.\n    - `:stream_exists` - Ensure the stream exists.\n\n  - `events_or_event_ids` is a list of `%EventStore.EventData{}` structs or\n    event ids.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database transaction, in\n      milliseconds. Defaults to 15,000ms.\n\nReturns `:ok` on success, or an `{:error, reason}` tagged tuple. The returned\nerror may be due to one of the following reasons:\n\n  * `{:error, :wrong_expected_version}` when the actual stream version differs\n    from the provided expected version.\n  * `{:error, :stream_exists}` when the stream exists, but expected version\n    was `:no_stream`.\n  * `{:error, :stream_not_found}` when the stream does not exist, but\n    expected version was `:stream_exists`.","ref":"EventStore.html#c:link_to_stream/4"},{"type":"callback","title":"EventStore.paginate_streams/1","doc":"Paginate all streams.\n\n  - `opts` an optional keyword list containing:\n\n    - `page_size` the total number of streams per page. Defaults to 50.\n\n    - `page_number` the current page number. Defaults to page 1.\n\n    - `search` search for a stream by its identity.\n\n    - `sort_by` sort the streams by the given field.\n      Defaults to sorting by the stream's internal id (`:stream_id` field)\n\n    - `sort_dir` direction to sort streams by, either `:asc` or `:desc`.\n      Defaults to `:asc`.\n\n    - `name` the name of the event store if provided to `start_link/1`.\n      Defaults to the event store module name (e.g. `MyApp.EventStore`).\n\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.\n\nReturns an `{:ok, page}` result containing a list of `StreamInfo` structs, or\nan error tagged tuple on failure.\n\n#","ref":"EventStore.html#c:paginate_streams/1"},{"type":"callback","title":"Example - EventStore.paginate_streams/1","doc":"alias EventStore.Page\n\n    {:ok, %Page{entries: streams}} = MyApp.EventStore.paginate_streams()","ref":"EventStore.html#c:paginate_streams/1-example"},{"type":"callback","title":"EventStore.read_all_streams_backward/3","doc":"Reads the requested number of events from all streams in the reverse order\nfrom which they were originally written.\n\n  - `start_version` optionally, the stream version of the first event to read.\n    Use `-1` to indicate starting from the end of the stream. Defaults to the\n    end of the stream if not set.\n\n  - `count` optionally, the maximum number of events to read.\n    Defaults to returning 1,000 events from all streams.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.","ref":"EventStore.html#c:read_all_streams_backward/3"},{"type":"callback","title":"EventStore.read_all_streams_forward/3","doc":"Reads the requested number of events from all streams in the order in which\nthey were originally written.\n\n  - `start_version` optionally, the stream version of the first event to read.\n    Defaults to the beginning of the stream if not set.\n\n  - `count` optionally, the maximum number of events to read.\n    Defaults to returning 1,000 events from all streams.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.","ref":"EventStore.html#c:read_all_streams_forward/3"},{"type":"callback","title":"EventStore.read_snapshot/2","doc":"Read a snapshot, if available, for a given source.\n\nReturns `{:ok, %EventStore.Snapshots.SnapshotData{}}` on success, or\n`{:error, :snapshot_not_found}` when unavailable.","ref":"EventStore.html#c:read_snapshot/2"},{"type":"callback","title":"EventStore.read_stream_backward/4","doc":"Reads the requested number of events from the given stream in the reverse\norder from which they were originally written.\n\n  - `stream_uuid` is used to uniquely identify a stream.\n\n  - `start_version` optionally, the stream version of the first event to read.\n    Use `-1` to indicate starting from the end of the stream. Defaults to the\n    end of the stream if not set.\n\n  - `count` optionally, the maximum number of events to read.\n    Defaults to to returning 1,000 events from the stream.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.","ref":"EventStore.html#c:read_stream_backward/4"},{"type":"callback","title":"EventStore.read_stream_forward/4","doc":"Reads the requested number of events from the given stream in the order in\nwhich they were originally written.\n\n  - `stream_uuid` is used to uniquely identify a stream.\n\n  - `start_version` optionally, the stream version of the first event to read.\n    Defaults to the beginning of the stream if not set.\n\n  - `count` optionally, the maximum number of events to read.\n    Defaults to to returning 1,000 events from the stream.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.","ref":"EventStore.html#c:read_stream_forward/4"},{"type":"callback","title":"EventStore.record_snapshot/2","doc":"Record a snapshot of the data and metadata for a given source.\n\nReturns `:ok` on success.","ref":"EventStore.html#c:record_snapshot/2"},{"type":"callback","title":"EventStore.start_link/1","doc":"Starts any connection pooling or supervision and return `{:ok, pid}`\nor just `:ok` if nothing needs to be done.\n\nReturns `{:error, {:already_started, pid}}` if the event store is already\nstarted or `{:error, term}` in case anything else goes wrong.","ref":"EventStore.html#c:start_link/1"},{"type":"callback","title":"EventStore.stop/2","doc":"Shuts down the event store.","ref":"EventStore.html#c:stop/2"},{"type":"callback","title":"EventStore.stream_all_backward/2","doc":"Streams events from all streams in the reverse order from which they were\noriginally written.\n\n  - `start_version` optionally, the stream version of the first event to read.\n    Use `-1` to indicate starting from the end of the stream. Defaults to the\n    end of the stream if not set.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.\n    - `read_batch_size` optionally, the number of events to read at a time from\n      storage. Defaults to reading 1,000 events per batch.","ref":"EventStore.html#c:stream_all_backward/2"},{"type":"callback","title":"EventStore.stream_all_forward/2","doc":"Streams events from all streams in the order in which they were originally\nwritten.\n\n  - `start_version` optionally, the stream version of the first event to read.\n    Defaults to the beginning of the stream if not set.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.\n    - `read_batch_size` optionally, the number of events to read at a time from\n      storage. Defaults to reading 1,000 events per batch.","ref":"EventStore.html#c:stream_all_forward/2"},{"type":"callback","title":"EventStore.stream_backward/3","doc":"Streams events from the given stream in the reverse order from which they\nwere originally written.\n\n  - `start_version` optionally, the stream version of the first event to read.\n    Use `-1` to indicate starting from the end of the stream. Defaults to the\n    end of the stream if not set.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.\n    - `read_batch_size` optionally, the number of events to read at a time\n      from storage. Defaults to reading 1,000 events per batch.","ref":"EventStore.html#c:stream_backward/3"},{"type":"callback","title":"EventStore.stream_forward/3","doc":"Streams events from the given stream in the order in which they were\noriginally written.\n\n  - `start_version` optionally, the stream version of the first event to read.\n    Defaults to the beginning of the stream if not set.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.\n    - `read_batch_size` optionally, the number of events to read at a time\n      from storage. Defaults to reading 1,000 events per batch.","ref":"EventStore.html#c:stream_forward/3"},{"type":"callback","title":"EventStore.stream_info/2","doc":"Get basic information about a stream, including its version, status, and\ncreated date.\n\n  - `opts` an optional keyword list containing:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `timeout` an optional timeout for the database query, in milliseconds.\n      Defaults to 15,000ms.\n\nReturns `{:ok, StreamInfo.t()}` on success, or an `{:error, reason}` tagged\ntuple. The returned error may be due to one of the following reasons:\n\n  * `{:error, :stream_not_found}` when the stream does not exist.\n  * `{:error, :stream_deleted}` when the stream was soft deleted.\n\n#","ref":"EventStore.html#c:stream_info/2"},{"type":"callback","title":"Example - EventStore.stream_info/2","doc":"alias EventStore.Streams.StreamInfo\n\n    {:ok, %StreamInfo{stream_version: stream_version}} =\n      MyApp.EventStore.stream_info(\"stream-1234\")","ref":"EventStore.html#c:stream_info/2-example"},{"type":"callback","title":"EventStore.subscribe/2","doc":"Create a transient subscription to a given stream.\n\n  - `stream_uuid` is the stream to subscribe to.\n    Use the `$all` identifier to subscribe to events from all streams.\n\n  - `opts` is an optional keyword list providing additional subscription\n    configuration:\n    - `name` the name of the event store if provided to `start_link/1`.\n    - `selector` to define a function to filter each event, i.e. returns\n      only those elements for which fun returns a truthy value\n    - `mapper` to define a function to map each recorded event before sending\n      to the subscriber.\n\nThe calling process will be notified whenever new events are appended to\nthe given `stream_uuid`.\n\nAs the subscription is transient you do not need to acknowledge receipt of\neach event. The subscriber process will miss any events if it is restarted\nand resubscribes. If you need a persistent subscription with guaranteed\nat-least-once event delivery and back-pressure you should use\n`c:EventStore.subscribe_to_stream/4`.","ref":"EventStore.html#c:subscribe/2"},{"type":"callback","title":"Notification message - EventStore.subscribe/2","doc":"Events will be sent to the subscriber, in batches, as `{:events, events}`\nwhere events is a collection of `EventStore.RecordedEvent` structs.","ref":"EventStore.html#c:subscribe/2-notification-message"},{"type":"callback","title":"Example - EventStore.subscribe/2","doc":"{:ok, subscription} = EventStore.subscribe(stream_uuid)\n\n    # receive first batch of events\n    receive do\n      {:events, events} ->\n        IO.puts \"Received events: \" <> inspect(events)\n    end","ref":"EventStore.html#c:subscribe/2-example"},{"type":"callback","title":"EventStore.subscribe_to_all_streams/3","doc":"Create a subscription to all streams. By default the subscription is persistent.\n\nSee `c:EventStore.subscribe_to_stream/4` for options.","ref":"EventStore.html#c:subscribe_to_all_streams/3"},{"type":"callback","title":"Example - EventStore.subscribe_to_all_streams/3","doc":"{:ok, subscription} = EventStore.subscribe_to_all_streams(\"all_subscription\", self())\n\n    # wait for the subscription confirmation\n    receive do\n      {:subscribed, ^subscription} ->\n        IO.puts \"Successfully subscribed to all streams\"\n    end\n\n    receive do\n      {:events, events} ->\n        IO.puts \"Received events: \" <> inspect(events)\n\n        # acknowledge receipt\n        EventStore.ack(subscription, events)\n    end","ref":"EventStore.html#c:subscribe_to_all_streams/3-example"},{"type":"callback","title":"EventStore.subscribe_to_stream/4","doc":"Create a subscription to a single stream. By default the subscription is persistent.\n\nThe `subscriber` process will be notified of each batch of events appended to\nthe single stream identified by `stream_uuid`.\n\n  - `stream_uuid` is the stream to subscribe to.\n    Use the `$all` identifier to subscribe to events from all streams.\n\n  - `subscription_name` is used to uniquely identify the subscription.\n\n  - `subscriber` is a process that will be sent `{:events, events}`\n    notification messages.\n\n  - `opts` is an optional keyword list providing additional subscription\n    configuration:\n\n    - `name` the name of the event store if provided to `start_link/1`.\n\n    - `start_from` is a pointer to the first event to receive.\n      It must be one of:\n        - `:origin` for all events from the start of the stream (default).\n        - `:current` for any new events appended to the stream after the\n          subscription has been created.\n        - any positive integer for a stream version to receive events after.\n\n    - `selector` to define a function to filter each event, i.e. returns\n      only those elements for which the function returns a truthy value.\n\n    - `mapper` to define a function to map each recorded event before sending\n      to the subscriber.\n\n    - `concurrency_limit` defines the maximum number of concurrent subscribers\n      allowed to connect to the subscription. By default only one subscriber\n      may connect. If too many subscribers attempt to connect to the\n      subscription an `{:error, :too_many_subscribers}` is returned.\n\n    - `buffer_size` limits how many in-flight events will be sent to the\n      subscriber process before acknowledgement of successful processing. This\n      limits the number of messages sent to the subscriber and stops their\n      message queue from getting filled with events. Defaults to one in-flight\n      event.\n\n    - `checkpoint_threshold` determines how frequently a checkpoint is written\n      to the database for the subscription after events are acknowledged.\n      Increasing the threshold will reduce the number of database writes for\n      busy subscriptions, but means that events might be replayed when the\n      subscription resumes if the checkpoint cannot be written.\n      The default is to persist the checkpoint after each acknowledgement.\n\n    - `checkpoint_after` (milliseconds) used to ensure a checkpoint is written\n      after a period of inactivity even if the checkpoint threshold has not\n      been met. This ensures checkpoints are consistently written during\n      less busy periods. It is only applicable when a checkpoint threshold has\n      been set as the default subscription behaviour is to checkpoint after\n      each acknowledgement.\n\n    - `max_size` limits the number of events queued in memory by the\n      subscription process to prevent excessive memory usage. If the in-memory\n      queue exceeds the max size - because the subscriber cannot keep up -\n      then events will not be queued in memory, but instead will be read from\n      the database on demand once the subscriber process has processed the\n      queue. This limit also determines how many events are read from the\n      database at a time during catch-up. Defaults to 1,000 events.\n\n    - `partition_by` is an optional function used to partition events to\n      subscribers. It can be used to guarantee processing order when multiple\n      subscribers have subscribed to a single subscription. The function is\n      passed a single argument (an `EventStore.RecordedEvent` struct) and must\n      return the partition key. As an example to guarantee events for a single\n      stream are processed serially, but different streams are processed\n      concurrently, you could use the `stream_uuid` as the partition key.\n\n        ```\n        by_stream = fn %EventStore.RecordedEvent{stream_uuid: stream_uuid} -> stream_uuid end\n\n        {:ok, _subscription} =\n          EventStore.subscribe_to_stream(stream_uuid, \"example\", self(),\n            concurrency_limit: 10,\n            partition_by: by_stream\n          )\n        ```\n\n    - `timeout` an optional timeout for database queries, in milliseconds.\n      Defaults to 15,000ms.\n\n    - `transient` is an optional boolean flag to create a transient subscription.\n      By default this is set to `false`. If you want to create a transient\n      subscription set this flag to true. Your subscription will not be\n      persisted, so if the subscription is restarted, you will receive the events\n      again starting from `start_from`.\n\n      An example usage are short lived event handlers that keep their state in\n      memory but still want to have the guarantee to have received all events.\n\n      It's possible to create a persistent subscription with some name,\n      stop it and later create a transient subscription with the same name. The\n      transient subscription will now receive all events starting from `start_from`.\n      If you later stop this `transient` subscription and start a persistent\n      subscription again with the same name, you will receive the events again\n      as if the transient subscription never existed.\n\nThe subscription will resume from the last acknowledged event if it already\nexists. It will ignore the `start_from` argument in this case.\n\nReturns `{:ok, subscription}` when subscription succeeds.","ref":"EventStore.html#c:subscribe_to_stream/4"},{"type":"callback","title":"Notification messages - EventStore.subscribe_to_stream/4","doc":"Subscribers will initially receive a `{:subscribed, subscription}` message\nonce the subscription has successfully subscribed.\n\nAfter this message events will be sent to the subscriber, in batches, as\n`{:events, events}` where events is a collection of `EventStore.RecordedEvent`\nstructs.","ref":"EventStore.html#c:subscribe_to_stream/4-notification-messages"},{"type":"callback","title":"Example - EventStore.subscribe_to_stream/4","doc":"{:ok, subscription} = EventStore.subscribe_to_stream(stream_uuid, \"example\", self())\n\n    # wait for the subscription confirmation\n    receive do\n      {:subscribed, ^subscription} ->\n        IO.puts \"Successfully subscribed to stream: \" <> inspect(stream_uuid)\n    end\n\n    receive do\n      {:events, events} ->\n        IO.puts \"Received events: \" <> inspect(events)\n\n        # acknowledge receipt\n        EventStore.ack(subscription, events)\n    end","ref":"EventStore.html#c:subscribe_to_stream/4-example"},{"type":"callback","title":"Subscription tuning - EventStore.subscribe_to_stream/4","doc":"Use the `checkpoint_threshold` and `checkpoint_after` options to configure how\nfrequently checkpoints are written to the database. By default a subscription\nwill persist a checkpoint after each acknowledgement. This can cause high\nwrite load on the database for busy subscriptions which receive a large number\nof events. This problem is known as write amplification where each event\nwritten to a stream causes many additional writes as subscriptions acknowledge\nprocessing of the event.\n\nThe `checkpoint_threshold` controls how frequently checkpoints are persisted.\nIncreasing the threshold reduces the number of database writes. For example\nusing a threshold of 100 means that a checkpoint is written at most once for\nevery 100 events processed. The `checkpoint_after` ensures that a checkpoint\nwill still be written after a period of inactivity even when the threshold has\nnot been met. This ensures bursts of event processing can be safely handled.","ref":"EventStore.html#c:subscribe_to_stream/4-subscription-tuning"},{"type":"callback","title":"EventStore.unsubscribe_from_all_streams/2","doc":"Unsubscribe an existing subscriber from all event notifications.\n\n  - `subscription_name` is used to identify the existing subscription process\n    to stop.\n\nReturns `:ok` on success.","ref":"EventStore.html#c:unsubscribe_from_all_streams/2"},{"type":"callback","title":"EventStore.unsubscribe_from_stream/3","doc":"Unsubscribe an existing subscriber from event notifications.\n\n  - `stream_uuid` is the stream to unsubscribe from.\n\n  - `subscription_name` is used to identify the existing subscription process\n    to stop.\n\nReturns `:ok` on success.","ref":"EventStore.html#c:unsubscribe_from_stream/3"},{"type":"type","title":"EventStore.expected_version/0","doc":"","ref":"EventStore.html#t:expected_version/0"},{"type":"type","title":"EventStore.option/0","doc":"","ref":"EventStore.html#t:option/0"},{"type":"type","title":"EventStore.options/0","doc":"","ref":"EventStore.html#t:options/0"},{"type":"type","title":"EventStore.pagination_option/0","doc":"","ref":"EventStore.html#t:pagination_option/0"},{"type":"type","title":"EventStore.pagination_options/0","doc":"","ref":"EventStore.html#t:pagination_options/0"},{"type":"type","title":"EventStore.persistent_subscription_option/0","doc":"","ref":"EventStore.html#t:persistent_subscription_option/0"},{"type":"type","title":"EventStore.persistent_subscription_options/0","doc":"","ref":"EventStore.html#t:persistent_subscription_options/0"},{"type":"type","title":"EventStore.start_from/0","doc":"","ref":"EventStore.html#t:start_from/0"},{"type":"type","title":"EventStore.t/0","doc":"","ref":"EventStore.html#t:t/0"},{"type":"type","title":"EventStore.transient_subscribe_option/0","doc":"","ref":"EventStore.html#t:transient_subscribe_option/0"},{"type":"type","title":"EventStore.transient_subscribe_options/0","doc":"","ref":"EventStore.html#t:transient_subscribe_options/0"},{"type":"module","title":"EventStore.Config","doc":"Provides access to the EventStore configuration.","ref":"EventStore.Config.html"},{"type":"function","title":"EventStore.Config.advisory_locks_postgrex_opts/1","doc":"Stop the Postgrex process when the database connection is lost.\n\nStopping the process allows a subscription to be notified when it has lost its\nadvisory lock.","ref":"EventStore.Config.html#advisory_locks_postgrex_opts/1"},{"type":"function","title":"EventStore.Config.column_data_type/2","doc":"Get the data type used to store event data and metadata.\n\nSupported data types are:\n\n  - \"bytea\" - Allows storage of binary strings.\n  - \"jsonb\" - Native JSON type, data is stored in a decomposed binary format\n    that makes it slightly slower to input due to added conversion overhead,\n    but significantly faster to process, since no reparsing is needed.","ref":"EventStore.Config.html#column_data_type/2"},{"type":"function","title":"EventStore.Config.default_postgrex_opts/1","doc":"","ref":"EventStore.Config.html#default_postgrex_opts/1"},{"type":"function","title":"EventStore.Config.get/2","doc":"Get the event store configuration for the environment.","ref":"EventStore.Config.html#get/2"},{"type":"function","title":"EventStore.Config.get_pool/0","doc":"Get the connection pool module for postgrex.","ref":"EventStore.Config.html#get_pool/0"},{"type":"function","title":"EventStore.Config.parse/1","doc":"Normalizes the event store configuration.","ref":"EventStore.Config.html#parse/1"},{"type":"function","title":"EventStore.Config.parsed/2","doc":"Get the event store configuration for the environment.","ref":"EventStore.Config.html#parsed/2"},{"type":"function","title":"EventStore.Config.postgrex_notifications_opts/2","doc":"","ref":"EventStore.Config.html#postgrex_notifications_opts/2"},{"type":"function","title":"EventStore.Config.postgrex_opts/2","doc":"","ref":"EventStore.Config.html#postgrex_opts/2"},{"type":"module","title":"EventStore.Notifications.Publisher.State","doc":"","ref":"EventStore.Notifications.Publisher.State.html"},{"type":"function","title":"EventStore.Notifications.Publisher.State.new/1","doc":"","ref":"EventStore.Notifications.Publisher.State.html#new/1"},{"type":"module","title":"EventStore.Page","doc":"A page of results from a paginated query.","ref":"EventStore.Page.html"},{"type":"function","title":"EventStore.Page.total_pages/2","doc":"","ref":"EventStore.Page.html#total_pages/2"},{"type":"type","title":"EventStore.Page.t/0","doc":"","ref":"EventStore.Page.html#t:t/0"},{"type":"type","title":"EventStore.Page.t/1","doc":"","ref":"EventStore.Page.html#t:t/1"},{"type":"module","title":"EventStore.PubSub","doc":"Pub/sub using Elixir's local `Registry` module.","ref":"EventStore.PubSub.html"},{"type":"function","title":"EventStore.PubSub.broadcast/3","doc":"Broadcasts message on given topic.","ref":"EventStore.PubSub.html#broadcast/3"},{"type":"function","title":"EventStore.PubSub.child_spec/1","doc":"Return the child spec.","ref":"EventStore.PubSub.html#child_spec/1"},{"type":"function","title":"EventStore.PubSub.subscribe/3","doc":"Subscribes the caller to the given topic.","ref":"EventStore.PubSub.html#subscribe/3"},{"type":"module","title":"EventStore.Streams.StreamInfo","doc":"","ref":"EventStore.Streams.StreamInfo.html"},{"type":"function","title":"EventStore.Streams.StreamInfo.new/1","doc":"","ref":"EventStore.Streams.StreamInfo.html#new/1"},{"type":"function","title":"EventStore.Streams.StreamInfo.validate_expected_version/2","doc":"","ref":"EventStore.Streams.StreamInfo.html#validate_expected_version/2"},{"type":"type","title":"EventStore.Streams.StreamInfo.t/0","doc":"","ref":"EventStore.Streams.StreamInfo.html#t:t/0"},{"type":"module","title":"EventStore.Tasks.Migration","doc":"","ref":"EventStore.Tasks.Migration.html"},{"type":"module","title":"EventStore.Tasks.Migrations","doc":"Task to show the migration status of EventStore","ref":"EventStore.Tasks.Migrations.html"},{"type":"function","title":"EventStore.Tasks.Migrations.exec/2","doc":"Run task","ref":"EventStore.Tasks.Migrations.html#exec/2"},{"type":"function","title":"Parameters - EventStore.Tasks.Migrations.exec/2","doc":"- config: the parsed EventStore config","ref":"EventStore.Tasks.Migrations.html#exec/2-parameters"},{"type":"function","title":"Opts - EventStore.Tasks.Migrations.exec/2","doc":"- is_mix: set to `true` if running as part of a Mix task","ref":"EventStore.Tasks.Migrations.html#exec/2-opts"},{"type":"module","title":"Mix.EventStore","doc":"Conveniences for writing EventStore related Mix tasks.","ref":"Mix.EventStore.html"},{"type":"function","title":"Mix.EventStore.ensure_event_store!/2","doc":"Ensures the given module is an EventStore.","ref":"Mix.EventStore.html#ensure_event_store!/2"},{"type":"function","title":"Mix.EventStore.implements?/2","doc":"Returns `true` if module implements behaviour.","ref":"Mix.EventStore.html#implements?/2"},{"type":"function","title":"Mix.EventStore.parse_event_store/1","doc":"Parses the event store option from the given command line args list.\n\nIf no event store option is given, it is retrieved from the application\nenvironment.","ref":"Mix.EventStore.html#parse_event_store/1"},{"type":"task","title":"mix event_store.create","doc":"Create the database for the EventStore.\n\nThe event stores to create are the ones specified under the\n`:event_stores` option in the current app configuration. However,\nif the `-e` option is given, it replaces the `:event_stores` config.","ref":"Mix.Tasks.EventStore.Create.html"},{"type":"task","title":"Examples - mix event_store.create","doc":"mix event_store.create -e MyApp.EventStore","ref":"Mix.Tasks.EventStore.Create.html#module-examples"},{"type":"task","title":"Command line options - mix event_store.create","doc":"* `-e`, `--eventstore` - the event store to create\n  * `--quiet` - do not log output","ref":"Mix.Tasks.EventStore.Create.html#module-command-line-options"},{"type":"task","title":"mix event_store.drop","doc":"Drop the database for the EventStore.\n\nThe event stores to drop are the ones specified under the\n`:event_stores` option in the current app configuration. However,\nif the `-e` option is given, it replaces the `:event_stores` config.","ref":"Mix.Tasks.EventStore.Drop.html"},{"type":"task","title":"Examples - mix event_store.drop","doc":"mix event_store.drop -e MyApp.EventStore","ref":"Mix.Tasks.EventStore.Drop.html#module-examples"},{"type":"task","title":"Command line options - mix event_store.drop","doc":"* `-e`, `--eventstore` - the event store to drop\n  * `--quiet` - do not log output","ref":"Mix.Tasks.EventStore.Drop.html#module-command-line-options"},{"type":"task","title":"mix event_store.init","doc":"Initialize the database for the EventStore.\n\nThe event stores to initialize are the ones specified under the\n`:event_stores` option in the current app configuration. However,\nif the `-e` option is given, it replaces the `:event_stores` config.","ref":"Mix.Tasks.EventStore.Init.html"},{"type":"task","title":"Examples - mix event_store.init","doc":"mix event_store.init -e MyApp.EventStore","ref":"Mix.Tasks.EventStore.Init.html#module-examples"},{"type":"task","title":"Command line options - mix event_store.init","doc":"* `-e`, `--eventstore` - the event store to create\n  * `--quiet` - do not log output","ref":"Mix.Tasks.EventStore.Init.html#module-command-line-options"},{"type":"task","title":"mix event_store.migrate","doc":"Migrate an existing EventStore database.\n\nThe event stores to migrate are the ones specified under the\n`:event_stores` option in the current app configuration. However,\nif the `-e` option is given, it replaces the `:event_stores` config.","ref":"Mix.Tasks.EventStore.Migrate.html"},{"type":"task","title":"Examples - mix event_store.migrate","doc":"mix event_store.migrate -e MyApp.EventStore","ref":"Mix.Tasks.EventStore.Migrate.html#module-examples"},{"type":"task","title":"Command line options - mix event_store.migrate","doc":"* `-e`, `--eventstore` - the event store to create\n  * `--quiet` - do not log output","ref":"Mix.Tasks.EventStore.Migrate.html#module-command-line-options"},{"type":"task","title":"mix event_store.migrations","doc":"Show the migration status of an existing EventStore database.\n\nThe event stores to inspect are the ones specified under the\n`:event_stores` option in the current app configuration. However,\nif the `-e` option is given, it replaces the `:event_stores` config.","ref":"Mix.Tasks.EventStore.Migrations.html"},{"type":"task","title":"Examples - mix event_store.migrations","doc":"mix event_store.migrations -e MyApp.EventStore","ref":"Mix.Tasks.EventStore.Migrations.html#module-examples"},{"type":"task","title":"Command line options - mix event_store.migrations","doc":"* `-e`, `--eventstore` - the event store to create\n  * `--quiet` - do not log output","ref":"Mix.Tasks.EventStore.Migrations.html#module-command-line-options"},{"type":"module","title":"EventStore.EventData","doc":"EventData contains the data for a single event before being persisted to storage","ref":"EventStore.EventData.html"},{"type":"function","title":"EventStore.EventData.fetch/2","doc":"","ref":"EventStore.EventData.html#fetch/2"},{"type":"function","title":"EventStore.EventData.get_and_update/3","doc":"","ref":"EventStore.EventData.html#get_and_update/3"},{"type":"type","title":"EventStore.EventData.t/0","doc":"","ref":"EventStore.EventData.html#t:t/0"},{"type":"type","title":"EventStore.EventData.uuid/0","doc":"","ref":"EventStore.EventData.html#t:uuid/0"},{"type":"module","title":"EventStore.RecordedEvent","doc":"`EventStore.RecordedEvent` contains the persisted data and metadata for a\nsingle event.\n\nEvents are immutable once recorded.","ref":"EventStore.RecordedEvent.html"},{"type":"module","title":"Recorded event fields - EventStore.RecordedEvent","doc":"- `event_number` - position of the event within the stream.\n    This will be identical to the `stream_version` when fetching events from a\n    single stream. For the `$all` stream it will be the globally ordered event\n    number.\n  - `event_id` - a globally unique UUID to identify the event.\n  - `stream_uuid` - the original stream identity for the event.\n  - `stream_version` - the original version of the stream for the event.\n  - `correlation_id` - an optional UUID identifier used to correlate related\n    messages.\n  - `causation_id` - an optional UUID identifier used to identify which\n    message you are responding to.\n  - `data` - the deserialized event data.\n  - `metadata` - a deserialized map of event metadata.\n  - `created_at` - a `DateTime` (in UTC) indicating when the event was\n    created.","ref":"EventStore.RecordedEvent.html#module-recorded-event-fields"},{"type":"function","title":"EventStore.RecordedEvent.deserialize/2","doc":"","ref":"EventStore.RecordedEvent.html#deserialize/2"},{"type":"function","title":"EventStore.RecordedEvent.fetch/2","doc":"","ref":"EventStore.RecordedEvent.html#fetch/2"},{"type":"function","title":"EventStore.RecordedEvent.get_and_update/3","doc":"","ref":"EventStore.RecordedEvent.html#get_and_update/3"},{"type":"type","title":"EventStore.RecordedEvent.t/0","doc":"","ref":"EventStore.RecordedEvent.html#t:t/0"},{"type":"type","title":"EventStore.RecordedEvent.uuid/0","doc":"","ref":"EventStore.RecordedEvent.html#t:uuid/0"},{"type":"module","title":"EventStore.Snapshots.SnapshotData","doc":"Snapshot data.","ref":"EventStore.Snapshots.SnapshotData.html"},{"type":"function","title":"EventStore.Snapshots.SnapshotData.deserialize/2","doc":"","ref":"EventStore.Snapshots.SnapshotData.html#deserialize/2"},{"type":"function","title":"EventStore.Snapshots.SnapshotData.serialize/2","doc":"","ref":"EventStore.Snapshots.SnapshotData.html#serialize/2"},{"type":"type","title":"EventStore.Snapshots.SnapshotData.t/0","doc":"","ref":"EventStore.Snapshots.SnapshotData.html#t:t/0"},{"type":"module","title":"EventStore.JsonSerializer","doc":"A serializer that uses the JSON format.","ref":"EventStore.JsonSerializer.html"},{"type":"function","title":"EventStore.JsonSerializer.deserialize/2","doc":"Deserialize given JSON binary data to the expected type.","ref":"EventStore.JsonSerializer.html#deserialize/2"},{"type":"function","title":"EventStore.JsonSerializer.serialize/1","doc":"Serialize given term to JSON binary data.","ref":"EventStore.JsonSerializer.html#serialize/1"},{"type":"module","title":"EventStore.JsonbSerializer","doc":"Serialize to/from PostgreSQL's native `jsonb` format.","ref":"EventStore.JsonbSerializer.html"},{"type":"function","title":"EventStore.JsonbSerializer.deserialize/2","doc":"","ref":"EventStore.JsonbSerializer.html#deserialize/2"},{"type":"function","title":"EventStore.JsonbSerializer.serialize/1","doc":"","ref":"EventStore.JsonbSerializer.html#serialize/1"},{"type":"function","title":"EventStore.JsonbSerializer.to_struct/2","doc":"","ref":"EventStore.JsonbSerializer.html#to_struct/2"},{"type":"behaviour","title":"EventStore.Serializer","doc":"Specification of a serializer to convert between an Elixir term and its\nrepresentation in the database.","ref":"EventStore.Serializer.html"},{"type":"callback","title":"EventStore.Serializer.deserialize/2","doc":"Deserialize the given data to the corresponding term.","ref":"EventStore.Serializer.html#c:deserialize/2"},{"type":"callback","title":"EventStore.Serializer.serialize/1","doc":"Serialize the given term to a representation that can be stored by the\ndatabase.","ref":"EventStore.Serializer.html#c:serialize/1"},{"type":"function","title":"EventStore.Serializer.serializer/2","doc":"Get the serializer module from the given config for the event store.","ref":"EventStore.Serializer.html#serializer/2"},{"type":"type","title":"EventStore.Serializer.config/0","doc":"","ref":"EventStore.Serializer.html#t:config/0"},{"type":"type","title":"EventStore.Serializer.t/0","doc":"","ref":"EventStore.Serializer.html#t:t/0"},{"type":"module","title":"EventStore.TermSerializer","doc":"A serializer that uses Erlang's external term format (http://erlang.org/doc/apps/erts/erl_ext_dist.html)","ref":"EventStore.TermSerializer.html"},{"type":"function","title":"EventStore.TermSerializer.deserialize/2","doc":"Deserialize given binary data in Erlang's external term format.","ref":"EventStore.TermSerializer.html#deserialize/2"},{"type":"function","title":"EventStore.TermSerializer.serialize/1","doc":"Serialize given term to binary data.","ref":"EventStore.TermSerializer.html#serialize/1"},{"type":"module","title":"EventStore.Tasks.Create","doc":"Task to create the EventStore","ref":"EventStore.Tasks.Create.html"},{"type":"function","title":"EventStore.Tasks.Create.exec/2","doc":"Runs database and schema create task.","ref":"EventStore.Tasks.Create.html#exec/2"},{"type":"function","title":"Parameters - EventStore.Tasks.Create.exec/2","doc":"- config: the parsed EventStore config","ref":"EventStore.Tasks.Create.html#exec/2-parameters"},{"type":"function","title":"Opts - EventStore.Tasks.Create.exec/2","doc":"- is_mix: set to `true` if running as part of a Mix task\n  - quiet: set to `true` to silence output","ref":"EventStore.Tasks.Create.html#exec/2-opts"},{"type":"module","title":"EventStore.Tasks.Drop","doc":"Task to drop the EventStore database.","ref":"EventStore.Tasks.Drop.html"},{"type":"function","title":"EventStore.Tasks.Drop.exec/2","doc":"Runs task","ref":"EventStore.Tasks.Drop.html#exec/2"},{"type":"function","title":"Parameters - EventStore.Tasks.Drop.exec/2","doc":"- config: the parsed EventStore config","ref":"EventStore.Tasks.Drop.html#exec/2-parameters"},{"type":"function","title":"Opts - EventStore.Tasks.Drop.exec/2","doc":"- is_mix: set to `true` if running as part of a Mix task\n  - quiet: set to `true` to silence output","ref":"EventStore.Tasks.Drop.html#exec/2-opts"},{"type":"module","title":"EventStore.Tasks.Init","doc":"Task to initalize the EventStore database","ref":"EventStore.Tasks.Init.html"},{"type":"function","title":"EventStore.Tasks.Init.exec/2","doc":"Runs task","ref":"EventStore.Tasks.Init.html#exec/2"},{"type":"function","title":"Parameters - EventStore.Tasks.Init.exec/2","doc":"- config: the parsed EventStore config","ref":"EventStore.Tasks.Init.html#exec/2-parameters"},{"type":"function","title":"Opts - EventStore.Tasks.Init.exec/2","doc":"- is_mix: set to `true` if running as part of a Mix task\n  - quiet: set to `true` to silence output","ref":"EventStore.Tasks.Init.html#exec/2-opts"},{"type":"module","title":"EventStore.Tasks.Migrate","doc":"Task to migrate EventStore","ref":"EventStore.Tasks.Migrate.html"},{"type":"function","title":"EventStore.Tasks.Migrate.exec/2","doc":"Run task","ref":"EventStore.Tasks.Migrate.html#exec/2"},{"type":"function","title":"Parameters - EventStore.Tasks.Migrate.exec/2","doc":"- config: the parsed EventStore config","ref":"EventStore.Tasks.Migrate.html#exec/2-parameters"},{"type":"function","title":"Opts - EventStore.Tasks.Migrate.exec/2","doc":"- is_mix: set to `true` if running as part of a Mix task\n  - quiet: set to `true` to silence output","ref":"EventStore.Tasks.Migrate.html#exec/2-opts"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.4.3 - Changelog","doc":"#","ref":"changelog.html#v1-4-3"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support Elixir v1.11 and later.\n* Parse url with encoded hash in password ([#275](https://github.com/commanded/eventstore/pull/275)).\n* Allow configuring the default database ([#277](https://github.com/commanded/eventstore/pull/277)).\n* Fix Elixir `Logger.warn/2` warning deprecation message ([#278](https://github.com/commanded/eventstore/pull/278)).\n* Quote schema names in SQL ([#266](https://github.com/commanded/eventstore/pull/266)).\n* Fix unused read batch size ([#279](https://github.com/commanded/eventstore/pull/279)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.4.2 - Changelog","doc":"#","ref":"changelog.html#v1-4-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Allow `disconnect_on_error_codes` to be passed to the event store Postgrex connection ([#263](https://github.com/commanded/eventstore/pull/263)).\n* Support all `Postgrex` and `DBConnection` options when configuring an event store ([#273](https://github.com/commanded/eventstore/pull/273)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Dialyzer error when calling `EventStore.subscribe/2` with `:name` option ([#268](https://github.com/commanded/eventstore/pull/268)).\n* Terminate a monitored process only if there are no other processes registered to use it ([#272](https://github.com/commanded/eventstore/pull/272)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.4.1 - Changelog","doc":"#","ref":"changelog.html#v1-4-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow `:parameters` to be passed to the EventStore database connection ([#257](https://github.com/commanded/eventstore/pull/257)).\n- Use configurable `:timeout` option for subscription database queries ([#259](https://github.com/commanded/eventstore/pull/259)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.4.0 - Changelog","doc":"- List running event store instances ([#244](https://github.com/commanded/eventstore/pull/244)).\n- Paginate streams ([#246](https://github.com/commanded/eventstore/pull/246)).\n- Add `stream_info/2` function ([#247](https://github.com/commanded/eventstore/pull/247)).\n- Remove unmaintained `elixir_uuid` dependency ([#253](https://github.com/commanded/eventstore/pull/253)).\n- Add option to use EventStore with PgBouncer ([#249](https://github.com/commanded/eventstore/pull/249)).\n\n---","ref":"changelog.html#v1-4-0"},{"type":"extras","title":"v1.3.2 - Changelog","doc":"#","ref":"changelog.html#v1-3-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add postgrex `socket_options` option ([#242](https://github.com/commanded/eventstore/pull/242)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix bug with subscriptions trigger in older Postgres versions ([#241](https://github.com/commanded/eventstore/pull/241)).\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Upgrading - Changelog","doc":"This release includes a database migration to be run. Please read the [Upgrading an EventStore](https://hexdocs.pm/eventstore/upgrades.html) guide for details on how to migrate an existing database.","ref":"changelog.html#upgrading"},{"type":"extras","title":"v1.3.1 - Changelog","doc":"#","ref":"changelog.html#v1-3-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Support running event store migrations when using a schema ([#239](https://github.com/commanded/eventstore/pull/239)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.3.0 - Changelog","doc":"- Improve performance of appending events under normal and degraded network conditions ([#230](https://github.com/commanded/eventstore/pull/230)).\n- Subscription checkpoint tuning ([#237](https://github.com/commanded/eventstore/pull/237)).","ref":"changelog.html#v1-3-0"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix bug with catch-up all streams subscription where the checkpoint is not committed for hard deleted streams ([#238](https://github.com/commanded/eventstore/pull/238)).\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Upgrading - Changelog","doc":"This release requires a database migration to be run. Please read the [Upgrading an EventStore](https://hexdocs.pm/eventstore/upgrades.html) guide for details on how to migrate an existing database.\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v1.2.3 - Changelog","doc":"- Add `:configure` to postgrex connection options ([#233](https://github.com/commanded/eventstore/pull/233)).\n- Use runtime configuration in Mix tasks ([#236](https://github.com/commanded/eventstore/pull/236)).","ref":"changelog.html#v1-2-3"},{"type":"extras","title":"v1.2.2 - Changelog","doc":"- Read stream and stream events backward ([#234](https://github.com/commanded/eventstore/pull/234)).","ref":"changelog.html#v1-2-2"},{"type":"extras","title":"v1.2.1 - Changelog","doc":"- Allow optional `event_id` to be included in `EventStore.EventData` struct ([#229](https://github.com/commanded/eventstore/pull/229)).\n- Adds an option to supply an existing database connection or transaction to `EventStore` functions ([#231](https://github.com/commanded/eventstore/pull/231)).","ref":"changelog.html#v1-2-1"},{"type":"extras","title":"v1.2.0 - Changelog","doc":"#","ref":"changelog.html#v1-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Delete event stream ([#203](https://github.com/commanded/eventstore/pull/203)).\n- Introduce `mix event_store.migrations` task to list migration status ([#207](https://github.com/commanded/eventstore/pull/207)).\n- Remove distributed registry ([#210](https://github.com/commanded/eventstore/pull/210)).\n- Hibernate subscription process after inactivity ([#214](https://github.com/commanded/eventstore/pull/214)).\n- Runtime event store configuration ([#217](https://github.com/commanded/eventstore/pull/217)).\n- Shared database connection pools ([#216](https://github.com/commanded/eventstore/pull/216)).\n- Shared database connection for notifications ([#225](https://github.com/commanded/eventstore/pull/225)).\n- Transient subscriptions ([#215](https://github.com/commanded/eventstore/pull/215))\n- Improve resilience when database connection is unavailable ([#226](https://github.com/commanded/eventstore/pull/226)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Upgrading - Changelog","doc":"This release requires a database migration to be run. Please read the [Upgrading an EventStore](https://hexdocs.pm/eventstore/upgrades.html) guide for details on how to migrate an existing database.\n\n#","ref":"changelog.html#upgrading"},{"type":"extras","title":"Breaking changes - Changelog","doc":"Usage of `EventStore.Tasks.Init` task to initialise an event store database has been changed as follows:\n\nPrevious usage:\n\n```elixir\n:ok = EventStore.Tasks.Init.exec(MyApp.EventStore, config, opts)\n```\n\nUsage now:\n\n```elixir\n:ok = EventStore.Tasks.Init.exec(config)\n:ok = EventStore.Tasks.Init.exec(config, opts)\n```\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Support appending events to a stream with `:any_version` concurrently ([#209](https://github.com/commanded/eventstore/pull/209)).\n\n---","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.1.0 - Changelog","doc":"#","ref":"changelog.html#v1-1-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support Postgres schemas ([#182](https://github.com/commanded/eventstore/pull/182)).\n- Dynamic event store ([#184](https://github.com/commanded/eventstore/pull/184)).\n- Add `timeout` option to config ([#189](https://github.com/commanded/eventstore/pull/189)).\n- Namespace advisory lock to prevent clash with other applications ([#166](https://github.com/commanded/eventstore/issues/166)).\n- Use database lock to prevent migrations from running concurrently ([#204](https://github.com/commanded/eventstore/pull/204)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"The following EventStore API functions have been changed where previously (in v1.0 and earlier) the last argument was an optional timeout (a non-negative integer or `:infinity`). This has been changed to be an optional Keyword list, which may include a timeout (e.g. `[timeout: 5_000]`). The `stream_forward` and `stream_all_forward` functions now also require the optional `read_batch_size` argument to be provided as part of the options Keyword list.\n\nThese changes were required to support dynamic event stores where an event store name can be included in the options to each function. If you did not provide a timeout to any of these functions then you will not need to make any changes to your code. See the example usages below for details.\n\n- `EventStore.append_to_stream`\n- `EventStore.link_to_stream`\n- `EventStore.read_stream_forward`\n- `EventStore.read_all_streams_forward`\n- `EventStore.stream_forward`\n- `EventStore.stream_all_forward`\n\nPrevious usage:\n\n```elixir\nEventStore.append_to_stream(stream_uuid, expected_version, events, timeout)\nEventStore.link_to_stream(stream_uuid, expected_version, events_or_event_ids, timeout)\nEventStore.read_stream_forward(stream_uuid, start_version, count, timeout)\nEventStore.read_all_streams_forward(start_version, count, timeout)\nEventStore.stream_forward(stream_uuid, start_version, read_batch_size, timeout)\nEventStore.stream_all_forward(start_version, read_batch_size, timeout)\n```\n\nUsage now:\n\n```elixir\nEventStore.append_to_stream(stream_uuid, expected_version, events, timeout: timeout)\nEventStore.link_to_stream(stream_uuid, expected_version, events_or_event_ids, timeout: timeout)\nEventStore.read_stream_forward(stream_uuid, start_version, count, timeout: timeout)\nEventStore.read_all_streams_forward(start_version, count, timeout: timeout)\nEventStore.stream_forward(stream_uuid, start_version, read_batch_size: read_batch_size, timeout: timeout)\nEventStore.stream_all_forward(start_version, read_batch_size: read_batch_size, timeout: timeout)\n```\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Upgrading - Changelog","doc":"This release requires a database migration to be run. Please read the [Upgrading an EventStore](https://hexdocs.pm/eventstore/upgrades.html) guide for details on how to migrate an existing database.\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v1.0.3 - Changelog","doc":"#","ref":"changelog.html#v1-0-3"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Use event's stream version when appending events to a stream ([#202](https://github.com/commanded/eventstore/pull/202)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.0.2 - Changelog","doc":"##","ref":"changelog.html#v1-0-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Prevent double supervision by starting / stopping supervisor manually ([#194](https://github.com/commanded/eventstore/pull/194)).\n- Use `DynamicSupervisor` for subscriptions.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v1.0.1 - Changelog","doc":"#","ref":"changelog.html#v1-0-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix `EventStore.Registration.DistributedForwarder` state when running multiple nodes ([#186](https://github.com/commanded/eventstore/pull/186)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"#","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support multiple event stores ([#168](https://github.com/commanded/eventstore/pull/168)).\n- Add support for `queue_target` and `queue_interval` database connection settings ([#172](https://github.com/commanded/eventstore/pull/172)).\n- Add support for `created_at` values to be of type `NaiveDateTime` ([#175](https://github.com/commanded/eventstore/pull/175)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix function clause error on `DBConnection.ConnectionError` ([#167](https://github.com/commanded/eventstore/issues/167)).\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Upgrading - Changelog","doc":"[Follow the upgrade guide](guides/upgrades/0.17-1.0.md) to define and use your own application specific event store].\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.17.0 - Changelog","doc":"#","ref":"changelog.html#v0-17-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- SSL support including Mix tasks ([#161](https://github.com/commanded/eventstore/pull/161)).\n- Use `timestamp with time zone` for timestamp fields ([#150](https://github.com/commanded/eventstore/pull/150)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Upgrading - Changelog","doc":"Upgrade your existing EventStore database by running:\n\n```console\nmix event_store.migrate\n```\n\n**Note**: The migrate command is idempotent and can be safely run multiple times.\n\nYou can drop and recreate an EventStore database by running:\n\n```console\nmix do event_store.drop, event_store.create, event_store.init\n```\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.16.2 - Changelog","doc":"#","ref":"changelog.html#v0-16-2"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix issue with concurrent subscription partitioning ([#162](https://github.com/commanded/eventstore/pull/162)).\n- Reliably start `EventStore.Notifications.Supervisor` on `:global` name clash ([#165](https://github.com/commanded/eventstore/pull/165)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.16.1 - Changelog","doc":"#","ref":"changelog.html#v0-16-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Stop Postgrex database connection process in mix `event_store.init` and `event_store.migrate` tasks after use to prevent IEx shutdown when tasks are run together (as `mix do event_store.init, event_store.migrate`).\n- Ensure the event store application doesn't crash when the database connection is lost ([#159](https://github.com/commanded/eventstore/pull/159)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.16.0 - Changelog","doc":"#","ref":"changelog.html#v0-16-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `:socket` and `:socket_dir` config options ([#132](https://github.com/commanded/eventstore/pull/132)).\n- Rename `uuid` dependency to `elixir_uuid` ([#135](https://github.com/commanded/eventstore/pull/135)).\n- Subscription concurrency ([#134](https://github.com/commanded/eventstore/pull/134)).\n- Send `:subscribed` message to all subscribers connected to a subscription ([#136](https://github.com/commanded/eventstore/pull/136)).\n- Update to `postgrex` v0.14 ([#143](https://github.com/commanded/eventstore/pull/143)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Replace `:poison` with `:jason` for JSON event data & metadata serialization ([#144](https://github.com/commanded/eventstore/pull/144)).\n\n  To support this change you will need to derive the `Jason.Encoder` protocol for all of your events.\n\n  This can be done by adding `@derive Jason.Encoder` before defining the struct in every event module.\n\n  ```elixir\n  defmodule Event1 do\n    @derive Jason.Encoder\n    defstruct [:id, :data]\n  end\n  ```\n\n  Or using `Protocol.derive/2` for each event, as shown below.\n\n  ```elixir\n  require Protocol\n\n  for event <- [Event1, Event2, Event3] do\n    Protocol.derive(Jason.Encoder, event)\n  end\n  ```\n\n---","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.15.1 - Changelog","doc":"#","ref":"changelog.html#0-15-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Use a timeout of `:infinity` for the migration task (`mix event_store.migrate`) to allow database migration to run longer than the default 15 seconds.\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Socket closing causes the event store to never receive notifications ([#130](https://github.com/commanded/eventstore/pull/130)).\n- Subscription with selector function should notify pending events after all filtered ([#131](https://github.com/commanded/eventstore/pull/131)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.15.0 - Changelog","doc":"- Support system environment variables for all config ([#115](https://github.com/commanded/eventstore/pull/115)).\n- Allow subscriptions to filter the events they receive ([#114](https://github.com/commanded/eventstore/pull/114)).\n- Allow callers to omit `event_type` when event data is a struct ([#118](https://github.com/commanded/eventstore/pull/118)).\n- Remove dependency on `psql` for `event_store.create`, `event_store.init`, `event_store.migrate`, and `event_store.drop` mix tasks ([#117](https://github.com/commanded/eventstore/pull/117)).\n- Supports query parameters in URL for database connection ([#119](https://github.com/commanded/eventstore/pull/119)).\n- Improve typespecs and include Dialyzer in Travis CI build ([#121](https://github.com/commanded/eventstore/pull/121)).\n\n---","ref":"changelog.html#0-15-0"},{"type":"extras","title":"0.14.0 - Changelog","doc":"- Add JSONB support ([#86](https://github.com/commanded/eventstore/pull/86)).\n- Add `:ssl` and `:ssl_opts` config params ([#88](https://github.com/commanded/eventstore/pull/88)).\n- Make `mix event_store.init` task do nothing if events table already exists ([#89](https://github.com/commanded/eventstore/pull/89)).\n- Timeout issue when using `EventStore.read_stream_forward` ([#92](https://github.com/commanded/eventstore/pull/92)).\n- Replace `:info` level logging with `:debug` ([#90](https://github.com/commanded/eventstore/issues/90)).\n- Dealing better with Poison dependency ([#91](https://github.com/commanded/eventstore/issues/91)).\n- Publish events directly to subscriptions ([#93](https://github.com/commanded/eventstore/pull/93)).\n- Use PostgreSQL advisory locks to enforce only one subscription instance ([#98](https://github.com/commanded/eventstore/pull/98)).\n- Remove stream process ([#99](https://github.com/commanded/eventstore/pull/99)).\n- Use PostgreSQL's `NOTIFY` / `LISTEN` for event pub/sub ([#100](https://github.com/commanded/eventstore/pull/100)).\n- Link existing events to another stream ([#103](https://github.com/commanded/eventstore/pull/103)).\n- Subscription notification message once successfully subscribed ([#104](https://github.com/commanded/eventstore/pull/104)).\n- Transient subscriptions ([#105](https://github.com/commanded/eventstore/pull/105)).\n- Transient subscription event mapping function ([#108](https://github.com/commanded/eventstore/pull/108)).\n- Turn EventStore `mix` tasks into generic tasks for use with Distillery during deployment ([#111](https://github.com/commanded/eventstore/pull/111)).\n\n#","ref":"changelog.html#0-14-0"},{"type":"extras","title":"Upgrading - Changelog","doc":"Upgrade your existing EventStore database by running:\n\n```console\nmix event_store.migrate\n```\n\nYou can drop and recreate an EventStore database by running:\n\n```console\nmix do event_store.drop, event_store.create, event_store.init\n```\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.13.2 - Changelog","doc":"#","ref":"changelog.html#v0-13-2"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Use `Supervisor.child_spec` with an explicit `id` for Registry processes to support Elixir v1.5.0 and v1.5.1 ([v1.5.2](https://github.com/elixir-lang/elixir/blob/v1.5/CHANGELOG.md#v152-2017-09-29) contains a fix for this issue).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.13.1 - Changelog","doc":"#","ref":"changelog.html#v0-13-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- EventStore migrate mix task read migration SQL scripts from app dir (`Application.app_dir(:eventstore)`).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.13.0 - Changelog","doc":"#","ref":"changelog.html#v0-13-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Use a UUID field for the `event_id` column, rename existing field to `event_number` ([#75](https://github.com/commanded/eventstore/issues/75)).\n- Use `uuid` data type for event `correlation_id` and `causation_id` ([#57](https://github.com/commanded/eventstore/pull/57)).\n- Mix task to migrate an existing EventStore database (`mix event_store.migrate`).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Append to stream is limited to 7,281 events in a single request ([#77](https://github.com/commanded/eventstore/issues/77)).\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Upgrading - Changelog","doc":"Upgrade your existing EventStore database by running: `mix event_store.migrate`\n\nOr you can drop and recreate the EventStore database by running: `mix do event_store.drop, event_store.create, event_store.init`\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.12.1 - Changelog","doc":"#","ref":"changelog.html#v0-12-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Publisher only notifies first pending event batch ([#81](https://github.com/commanded/eventstore/issues/81)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.12.0 - Changelog","doc":"#","ref":"changelog.html#v0-12-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow optimistic concurrency check on write to be optional ([#31](https://github.com/commanded/eventstore/issues/31)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix issue where subscription doesn't immediately receive events published while transitioning between catch-up and subscribed. Any missed events would be noticed and replayed upon next event publish.\n\n---","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"#","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support for running on a cluster of nodes using [Swarm](https://hex.pm/packages/swarm) for process distribution ([#53](https://github.com/commanded/eventstore/issues/53)).\n\n- Add `stream_version` column to `streams` table. It is used for stream info querying and optimistic concurrency checks, instead of querying the `events` table.\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Upgrading - Changelog","doc":"Run the schema migration [v0.11.0.sql](priv/event_store/migrations/v0.11.0.sql) script against your event store database.\n\n---","ref":"changelog.html#upgrading"},{"type":"extras","title":"v0.10.1 - Changelog","doc":"#","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix for ack of last seen event in stream subscription ([#66](https://github.com/commanded/eventstore/pull/66)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"#","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Writer per event stream ([#55](https://github.com/commanded/eventstore/issues/55)).\n\n  You **must** run the schema migration [v0.10.0.sql](priv/event_store/migrations/v0.10.0.sql) script against your event store database.\n\n- Use [DBConnection](https://hexdocs.pm/db_connection/DBConnection.html)'s built in support for connection pools (using poolboy).\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"#","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Adds `causation_id` alongside `correlation_id` for events ([#48](https://github.com/commanded/eventstore/pull/48)).\n\n  To migrate an existing event store database execute [v0.9.0.sql](priv/event_store/migrations/v0.9.0.sql) script.\n\n- Allow single stream, and all streams, subscriptions to provide a mapper function that maps every received event before sending to the subscriber.\n\n  ```elixir\n  EventStore.subscribe_to_stream(stream_uuid, \"subscription\", subscriber, mapper: fn event -> event.data end)\n  ```\n\n- Subscribers now receive an `{:events, events}` tuple and should acknowledge receipt by: `EventStore.ack(subscription, events)`\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.8.1 - Changelog","doc":"#","ref":"changelog.html#v0-8-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add Access functions to `EventStore.EventData` and `EventStore.RecordedEvent` modules ([#37](https://github.com/commanded/eventstore/pull/37)).\n- Allow database connection URL to be provided as a system variable ([#39](https://github.com/commanded/eventstore/pull/39)).\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Writer not parsing database connection URL from config ([#38](https://github.com/commanded/eventstore/pull/38/files)).","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"#","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Stream events from a single stream forward.\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.7.4 - Changelog","doc":"#","ref":"changelog.html#v0-7-4"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Subscriptions use Elixir [streams](https://hexdocs.pm/elixir/Stream.html) to read events when catching up.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.7.3 - Changelog","doc":"#","ref":"changelog.html#v0-7-3"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Upgrade `fsm` dependency to v0.3.0 to remove Elixir 1.4 compiler warnings.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.7.2 - Changelog","doc":"#","ref":"changelog.html#v0-7-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Stream all events forward ([#34](https://github.com/commanded/eventstore/issues/34)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.7.1 - Changelog","doc":"#","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow snapshots to be deleted ([#26](https://github.com/commanded/eventstore/issues/26)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"#","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Subscribe to a single stream, or all streams, from a specified start position ([#17](https://github.com/commanded/eventstore/issues/17)).\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.6.2 - Changelog","doc":"#","ref":"changelog.html#v0-6-2"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Subscriptions that are at max capacity should wait until all pending events have been acknowledged by the subscriber being catching up with any unseen events.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"#","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Use IO lists to build insert events SQL statement ([#23](https://github.com/commanded/eventstore/issues/23)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"#","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Use `NaiveDateTime` for each recorded event's `created_at` property.\n\n---","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"#","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Provide typespecs for the public API ([#16](https://github.com/commanded/eventstore/issues/16))\n- Fix compilation warnings in mix database task ([#14](https://github.com/commanded/eventstore/issues/14))\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Read stream forward does not use count to limit returned events ([#10](https://github.com/commanded/eventstore/issues/10))","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"#","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Ack handled events in subscribers ([#18](https://github.com/commanded/eventstore/issues/18)).\n- Buffer events between publisher and subscriber ([#19](https://github.com/commanded/eventstore/issues/19)).","ref":"changelog.html#enhancements"},{"type":"extras","title":"Getting started","doc":"# Getting started\n\nEventStore is [available in Hex](https://hex.pm/packages/eventstore) and can be installed as follows:\n\n  1. Add eventstore to your list of dependencies in `mix.exs`:\n\n      ```elixir\n      def deps do\n        [{:eventstore, \"~> 1.4\"}]\n      end\n      ```\n\n      Run `mix deps.get` to install the new dependency.\n\n  2. Define an event store module for your application:\n\n      ```elixir\n      defmodule MyApp.EventStore do\n        use EventStore, otp_app: :my_app\n\n        # Optional `init/1` function to modify config at runtime.\n        def init(config) do\n          {:ok, config}\n        end\n      end\n      ```\n\n  3. Add a config entry containing the PostgreSQL database connection details for your event store module to each environment's mix config file (e.g. `config/dev.exs`):\n\n      ```elixir\n      config :my_app, MyApp.EventStore,\n        serializer: EventStore.JsonSerializer,\n        username: \"postgres\",\n        password: \"postgres\",\n        database: \"eventstore\",\n        hostname: \"localhost\"\n\n      # OR use a URL to connect instead\n      config :my_app, MyApp.EventStore,\n        serializer: EventStore.JsonSerializer,\n        url: \"postgres://postgres:postgres@localhost/eventstore\"\n      ```\n\n\n      **Note:** Some managed database providers (such as DigitalOcean) don't provide access to the default `postgres` database. In such case, you can specify a default database in the following way:\n\n      ```elixir\n      config :my_app, MyApp.EventStore,\n        default_database: \"defaultdb\",\n      ```\n\n      **Note:** To use an EventStore with Commanded you should configure the event\n      store to use Commanded's JSON serializer which provides additional support for\n      JSON decoding:\n\n      ```elixir\n      config :my_app, MyApp.EventStore, serializer: Commanded.Serialization.JsonSerializer\n      ```\n\n      Configure optional database connection settings:\n\n      ```elixir\n      config :my_app, MyApp.EventStore,\n        pool_size: 10\n        queue_target: 50\n        queue_interval: 1_000,\n        schema: \"schema_name\"\n      ```\n\n      The database connection pool configuration options are:\n\n      - `:pool_size` - The number of connections (default: `10`).\n\n      Handling requests is done through a queue. When DBConnection is started, there are two relevant options to control the queue:\n\n      - `:queue_target` - in milliseconds (default: 50ms).\n      - `:queue_interval` - in milliseconds (default: 1,000ms).\n\n      Additional options:\n\n      - `:schema` - define the Postgres schema to use (default: `public` schema).\n      - `:timeout` - set the default database query timeout in milliseconds (default: 15,000ms).\n      - `:shared_connection_pool` - allows a database connection pool to be shared amongst multiple event store instances (default: `nil`).\n\n      Subscription options:\n\n      - `:subscription_retry_interval` - interval between subscription connection retry attempts (default: 60,000ms).\n      - `:subscription_hibernate_after` - subscriptions will automatically hibernate to save memory after a period of inactivity (default: 15,000ms).\n\n  4. Add your event store module to the `event_stores` list for your app in mix config:\n\n      ```elixir\n      # config/config.exs\n      config :my_app, event_stores: [MyApp.EventStore]\n      ```\n\n      This ensures the event store mix tasks can be run without specifying the event store as a command line argument (e.g. `mix event_store.init -e MyApp.EventStore`).\n\n  5. Create and initialize the event store database and tables using the `mix` tasks:\n\n      ```console\n      $ mix do event_store.create, event_store.init\n      ```\n\n  6. The final piece of configuration is to setup your event store as a supervisor within your application's supervision tree (e.g. in `lib/my_app/application.ex`, inside the `start/2` function):\n\n      ```elixir\n      defmodule MyApp.Application do\n        use Application\n\n        def start(_type, _args) do\n          children = [\n            MyApp.EventStore\n          ]\n\n          opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n          Supervisor.start_link(children, opts)\n        end\n      end\n      ```","ref":"getting-started.html"},{"type":"extras","title":"Using an existing database - Getting started","doc":"You can use an existing PostgreSQL database with EventStore by running the following `mix` task to create and initialize the event store tables:\n\n```console\n$ mix event_store.init\n```","ref":"getting-started.html#using-an-existing-database"},{"type":"extras","title":"Reset an existing database - Getting started","doc":"To drop an existing EventStore database and recreate it you can run the following `mix` tasks:\n\n```console\n$ mix do event_store.drop, event_store.create, event_store.init\n```\n\n*Warning* This will drop the database and delete all data.","ref":"getting-started.html#reset-an-existing-database"},{"type":"extras","title":"Initialize a database using an Elixir release - Getting started","doc":"If you're using an Elixir release build by the task [mix release](https://hexdocs.pm/mix/Mix.Tasks.Release.html) you won't have `mix` available therefore you won't be able to run the following command in order to initialize a new database.\n\n```console\n$ mix do event_store.create, event_store.init\n```\n\nTo do that you can use task modules defined inside EventStore (in `lib/mix/tasks`):\n\n* [EventStore.Tasks.Create](https://github.com/commanded/eventstore/blob/master/lib/event_store/tasks/create.ex)\n* [EventStore.Tasks.Init](https://github.com/commanded/eventstore/blob/master/lib/event_store/tasks/init.ex)\n\n So you can take advantage of the [running one-off commands](https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-one-off-commands-eval-and-rpc) supported by Mix release, using a helper module defined like this:\n\n```elixir\ndefmodule MyApp.ReleaseTasks do\n  def init_event_store do\n    {:ok, _} = Application.ensure_all_started(:postgrex)\n    {:ok, _} = Application.ensure_all_started(:ssl)\n\n    :ok = Application.load(:my_app)\n\n    config = MyApp.EventStore.config()\n\n    :ok = EventStore.Tasks.Create.exec(config, [])\n    :ok = EventStore.Tasks.Init.exec(config, [])\n  end\nend\n```","ref":"getting-started.html#initialize-a-database-using-an-elixir-release"},{"type":"extras","title":"Using Postgres schemas - Getting started","doc":"A Postgres database contains one or more [named schemas](https://www.postgresql.org/docs/current/ddl-schemas.html), which in turn contain tables. By default tables are defined in a schema named \"public\".\n\nAn EventStore can be configured to use a different schema name. Specify the schema when using the `EventStore` macro in your event store module:\n\n```elixir\ndefmodule MyApp.EventStore do\n  use EventStore, otp_app: :my_app, schema: \"example\"\nend\n```\n\nOr provide the schema as an option in the `init/1` callback function:\n\n```elixir\ndefmodule MyApp.EventStore do\n  use EventStore, otp_app: :my_app\n\n  def init(config) do\n    {:ok, Keyword.put(config, :schema, \"example\")}\n  end\nend\n```\n\nOr define it in environment config when configuring the database connection settings:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.EventStore, schema: \"example\"\n```\n\nThis feature allows you to define and start multiple event stores sharing a single Postgres database, but with their data isolated and segregated by schema.\n\nNote the `mix event_store. ` tasks to create, initialize, and drop an event store database will also handle creating and/or dropping the schema.","ref":"getting-started.html#using-postgres-schemas"},{"type":"extras","title":"Event data and metadata data type - Getting started","doc":"EventStore has support for persisting event data and metadata as either:\n\n  - Binary data, using the [`bytea` data type](https://www.postgresql.org/docs/current/static/datatype-binary.html), designed for storing binary strings.\n  - JSON data, using the [`jsonb` data type](https://www.postgresql.org/docs/current/static/datatype-json.html), specifically designed for storing JSON encoded data.\n\nThe default data type is `bytea`. This can be used with any binary serializer, such as the Erlang Term format, JSON data encoded to binary, and other serialization formats.\n\n#","ref":"getting-started.html#event-data-and-metadata-data-type"},{"type":"extras","title":"Using the `jsonb` data type - Getting started","doc":"The advantage this format is that it allows you to execute ad-hoc SQL queries against the event data or metadata using PostgreSQL's native JSON query support.\n\nTo enable native JSON support you need to configure your event store to use the `jsonb` data type. You must also use the `EventStore.JsonbSerializer` serializer, to ensure event data and metadata is correctly serialized to JSON, and include the Postgres types module (`EventStore.PostgresTypes`) for the Postgrex library to support JSON.\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.EventStore,\n  column_data_type: \"jsonb\",\n  serializer: EventStore.JsonbSerializer,\n  types: EventStore.PostgresTypes\n```\n\nFinally, you need to include the Jason library as a dependency in `mix.exs` to enable Postgrex JSON support and then run `mix deps.get` to install.\n\n```elixir\n# mix.exs\ndefp deps do\n  [{:jason, \"~> 1.2\"}]\nend\n```\n\nThese settings must be configured *before* creating the EventStore database. It's not possible to migrate between `bytea` and `jsonb` data types once you've created the database. This must be decided in advance.","ref":"getting-started.html#using-the-jsonb-data-type"},{"type":"extras","title":"Using with PgBouncer - Getting started","doc":"EventStore uses `LISTEN/NOTIFY` and `pg_advisory_locks` Postgres capabilities. Unfortunately, they are not compatible with PgBouncer running in transaction (most typical) mode.\n\nAs a workaround, you can provide an additional `session_mode_url` parameter to the EventStore config:\n\n```\nconfig :my_app, MyApp.EventStore,\n  url: \"postgres://postgres:pgbouncer-in-transaction-mode@localhost/eventstore\"\n  session_mode_url: \"postgres://postgres:pgbouncer-in-session-mode@localhost/eventstore\"\n```\n\nThis will allow the EventStore to use your regular pool settings to connect to the database defined in `url` for most database operations. It will separately establish connections using the `session_mode_url` where necessary which you should point to PgBouncer in session mode or connected directly to the Postgres instance.","ref":"getting-started.html#using-with-pgbouncer"},{"type":"extras","title":"Using the EventStore","doc":"# Using the EventStore\n\nFirst you need to define your own event store module:\n\n```elixir\ndefmodule MyApp.EventStore do\n  use EventStore, otp_app: :my_app\nend\n```","ref":"usage.html"},{"type":"extras","title":"Writing to a stream - Using the EventStore","doc":"Create a unique identity for each stream. It **must** be a string. This example uses the [elixir_uuid](https://hex.pm/packages/elixir_uuid) package.\n\n```elixir\nstream_uuid = EventStore.UUID.uuid4()\n```\n\nSet the expected version of the stream. This is used for optimistic concurrency. A new stream will be created when the expected version is zero.\n\n```elixir\nexpected_version = 0\n```\n\nBuild a list of events to persist. The data and metadata fields will be serialized to binary data. This uses your own serializer, as defined in config, that implements the `EventStore.Serializer` behaviour.\n\n```elixir\nalias EventStore.EventData\nalias MyApp.EventStore\n\ndefmodule ExampleEvent do\n  defstruct [:key]\nend\n\nevents = [\n  %EventData{\n    event_type: \"Elixir.ExampleEvent\",\n    data: %ExampleEvent{key: \"value\"},\n    metadata: %{user: \"someuser@example.com\"},\n  }\n]\n```\n\nAppend the events to the stream:\n\n```elixir\n:ok = EventStore.append_to_stream(stream_uuid, expected_version, events)\n```\n\n#","ref":"usage.html#writing-to-a-stream"},{"type":"extras","title":"Appending events to an existing stream - Using the EventStore","doc":"The expected version should equal the number of events already persisted to the stream when appending to an existing stream.\n\nThis can be set as the length of events returned from reading the stream:\n\n```elixir\nalias MyApp.EventStore\n\nevents =\n  stream_uuid\n  |> EventStore.stream_forward()\n  |> Enum.to_list()\n\nstream_version = length(events)\n```\n\nAppend new events to the existing stream:\n\n```elixir\nalias EventStore.EventData\nalias MyApp.EventStore\n\nnew_events = [ %EventData{..}, ... ]\n\n:ok = EventStore.append_to_stream(stream_uuid, stream_version, new_events)\n```\n\n##","ref":"usage.html#appending-events-to-an-existing-stream"},{"type":"extras","title":"Why must you provide the expected stream version? - Using the EventStore","doc":"This is to ensure that no events have been appended to the stream by another process between your read and subsequent write.\n\nThe `c:EventStore.append_to_stream/4` function will return `{:error, :wrong_expected_version}` when the version you provide is mismatched with the stream. You can resolve this error by reading the stream's events again, then attempt to append your new events using the latest stream version.\n\n#","ref":"usage.html#why-must-you-provide-the-expected-stream-version"},{"type":"extras","title":"Optional concurrency check - Using the EventStore","doc":"You can choose to append events to a stream without using the concurrency check, or having first read them from the stream, by using one of the following values instead of the expected version:\n\n- `:any_version` - No concurrency checking; allow any stream version (including no stream).\n- `:no_stream` - Ensure the stream does not exist.\n- `:stream_exists` - Ensure the stream exists.\n\n```elixir\nalias MyApp.EventStore\n\n:ok = EventStore.append_to_stream(stream_uuid, :any_version, events)\n```","ref":"usage.html#optional-concurrency-check"},{"type":"extras","title":"Reading from a stream - Using the EventStore","doc":"Read all events from a single stream, starting at the stream's first event:\n\n```elixir\nalias MyApp.EventStore\n\n{:ok, events} = EventStore.read_stream_forward(stream_uuid)\n```","ref":"usage.html#reading-from-a-stream"},{"type":"extras","title":"Reading from all streams - Using the EventStore","doc":"Read all events from all streams:\n\n```elixir\nalias MyApp.EventStore\n\n{:ok, events} = EventStore.read_all_streams_forward()\n```\n\nBy default this will be limited to read the first 1,000 events from all streams only.","ref":"usage.html#reading-from-all-streams"},{"type":"extras","title":"Stream from all streams - Using the EventStore","doc":"Stream all events from all streams:\n\n```elixir\nalias MyApp.EventStore\n\nall_events = EventStore.stream_all_forward() |> Enum.to_list()\n```\n\nThis will read *all* events into memory, it is for illustration only. Use the `Stream` functions to process the events in a memory efficient way.","ref":"usage.html#stream-from-all-streams"},{"type":"extras","title":"Linking events between streams - Using the EventStore","doc":"Event linking allows you to include events in multiple streams, such as copying an event from one stream to another, but only a reference to the original event is stored.\n\nAn event may be present in a stream only once, but may be linked into as many streams as required.\n\nLinked events are used to build the `$all` stream containing every persisted event, globally ordered.\n\nUse each recorded event's `event_number` field for the position of the event within the read/received stream. The `stream_uuid` and `stream_version` fields refer to the event's original stream.\n\nRead source events:\n\n```elixir\nalias MyApp.EventStore\n\n{:ok, events} = EventStore.read_stream_forward(source_stream_uuid)\n```\n\nLink read events to another stream:\n\n```elixir\nalias MyApp.EventStore\n\n:ok = EventStore.link_to_stream(target_stream_uuid, 0, events)\n```\n\nYou can also pass a list of `event_ids` instead of recorded event structs to link events.","ref":"usage.html#linking-events-between-streams"},{"type":"extras","title":"Subscriptions","doc":"# Subscriptions\n\nThere are two types of subscriptions provided by EventStore:\n\n1. [Transient subscriptions](#transient-subscriptions) where new events are broadcast to subscribers immediately after they have been appended to storage.\n2. [Persistent subscriptions](#persistent-subscriptions) which guarantee at-least-once delivery of every persisted event, provide back-pressure, and can be started, paused, and resumed from any position, including from the stream's origin.","ref":"subscriptions.html"},{"type":"extras","title":"Event pub/sub - Subscriptions","doc":"PostgreSQL's `LISTEN` and `NOTIFY` commands are used to pub/sub event notifications from the database. An after update trigger on the `streams` table is used to execute `NOTIFY` for each batch of inserted events. The notification payload contains the stream uuid, stream id, and first / last stream versions (e.g. `stream-12345,1,1,5`).\n\nA single process will connect to the database to listen for these notifications. It fetches the event data and broadcasts to all interested subscriptions. This approach supports running an EventStore on multiple nodes, regardless of whether they are connected together to form a cluster using distributed Erlang. One connection per node is used for single node and multi-node deployments.","ref":"subscriptions.html#event-pub-sub"},{"type":"extras","title":"Transient subscriptions - Subscriptions","doc":"Use `c:EventStore.subscribe/2` to create a transient subscription to a single stream identified by its `stream_uuid`. Events will be received in batches as an `{:events, events}` message, where `events` is a collection of `EventStore.RecordedEvent` structs.\n\nYou can use `$all` as the stream identity to subscribe to events appended to all streams. With transient subscriptions you do not need to acknowledge receipt of the published events. The subscription will terminate when the subscriber process stops running.\n\n##","ref":"subscriptions.html#transient-subscriptions"},{"type":"extras","title":"Subscribe to single stream events - Subscriptions","doc":"Subscribe to events appended to a *single* stream:\n\n```elixir\nalias MyApp.EventStore\n\n:ok = EventStore.subscribe(stream_uuid)\n\n# receive first batch of events\nreceive do\n  {:events, events} ->\n    IO.puts(\"Received events: \" <> inspect(events))\nend\n```\n\n##","ref":"subscriptions.html#subscribe-to-single-stream-events"},{"type":"extras","title":"Filtering events - Subscriptions","doc":"You can provide an event selector function that filters each `RecordedEvent` before sending it to the subscriber:\n\n```elixir\nalias EventStore.RecordedEvent\nalias MyApp.EventStore\n\nEventStore.subscribe(stream_uuid, selector: fn\n  %RecordedEvent{data: data} -> data != nil\nend)\n\n# receive first batch of mapped event data\nreceive do\n  {:events, %RecordedEvent{} = event_data} ->\n    IO.puts(\"Received non nil event data: \" <> inspect(event_data))\nend\n```\n\n##","ref":"subscriptions.html#filtering-events"},{"type":"extras","title":"Mapping events - Subscriptions","doc":"You can provide an event mapping function that maps each `RecordedEvent` before sending it to the subscriber:\n\n```elixir\nalias EventStore.RecordedEvent\nalias MyApp.EventStore\n\nEventStore.subscribe(stream_uuid, mapper: fn\n  %RecordedEvent{data: data} -> data\nend)\n\n# receive first batch of mapped event data\nreceive do\n  {:events, event_data} ->\n    IO.puts(\"Received event data: \" <> inspect(event_data))\nend\n```","ref":"subscriptions.html#mapping-events"},{"type":"extras","title":"Persistent subscriptions - Subscriptions","doc":"Persistent subscriptions to a stream will guarantee *at least once* delivery of every persisted event. Each subscription may be independently paused, then later resumed from where it stopped. The last received and acknowledged event is stored by the EventStore to support resuming at a later time or whenever the subscriber process restarts.\n\nA subscription can be created to receive events appended to a single or all streams.\n\nSubscriptions must be uniquely named. By default a subscription only supports a single subscriber. Attempting to connect two subscribers to the same subscription will return `{:error, :subscription_already_exists}`. You can optionally create a [competing consumer subscription with multiple subscribers](#subscription-concurrency).\n\n### `:subscribed` message\n\nOnce the subscription has successfully subscribed to the stream it will send the subscriber a `{:subscribed, subscription}` message. This indicates the subscription succeeded and you will begin receiving events.\n\nOnly one instance of a named subscription to a stream can connect to the database. This guarantees that starting the same subscription on each node when run on a cluster, or when running multiple single instance nodes, will only allow one subscription to actually connect. Therefore you can defer any initialisation until receipt of the `{:subscribed, subscription}` message to prevent duplicate effort by multiple nodes racing to create or subscribe to the same subscription.\n\n### `:events` message\n\nFor each batch of events appended to the event store your subscriber will receive a `{:events, events}` message. The `events` list is a collection of `EventStore.RecordedEvent` structs.\n\n#","ref":"subscriptions.html#persistent-subscriptions"},{"type":"extras","title":"Subscription start from - Subscriptions","doc":"By default subscriptions are created from the stream origin; they will receive all events from the stream. You can optionally specify a given start position:\n\n- `:origin` - subscribe to events from the start of the stream (identical to using `0`). This is the default behaviour.\n- `:current` - subscribe to events from the current version.\n- `event_number` (integer) - specify an exact event number to subscribe from. This will be the same as the stream version for single stream subscriptions.\n\n#","ref":"subscriptions.html#subscription-start-from"},{"type":"extras","title":"Acknowledge received events - Subscriptions","doc":"Receipt of each event by the subscriber must be acknowledged. This allows the subscription to resume on failure without missing an event and to indicate the subscription is ready to receive the next event.\n\nThe subscriber receives an `{:events, events}` tuple containing the published events. A subscription is returned when subscribing to the stream. This should be used to send the acknowledgement to using the `c:EventStore.ack/2` function:\n\n ```elixir\n alias MyApp.EventStore\n\n :ok = EventStore.ack(subscription, events)\n ```\n\nA subscriber can confirm receipt of each event in a batch by sending multiple acks, one per event. The subscriber may confirm receipt of the last event in the batch in a single ack.\n\nA subscriber will not receive further published events until it has confirmed receipt of all received events. This provides back pressure to the subscription to prevent the subscriber from being overwhelmed with messages if it cannot keep up. The subscription will buffer events until the subscriber is ready to receive, or an overflow occurs. At which point it will move into a catch-up mode and query events and replay them from storage until caught up.\n\n##","ref":"subscriptions.html#acknowledge-received-events"},{"type":"extras","title":"Subscribe to all events - Subscriptions","doc":"Subscribe to events appended to all streams:\n\n```elixir\nalias MyApp.EventStore\n\n{:ok, subscription} = EventStore.subscribe_to_all_streams(\"example_all_subscription\", self())\n\n# Wait for the subscription confirmation\nreceive do\n  {:subscribed, ^subscription} ->\n    IO.puts(\"Successfully subscribed to all streams\")\nend\n\nreceive do\n  {:events, events} ->\n    IO.puts \"Received events: #{inspect events}\"\n\n    # Acknowledge receipt\n    :ok = EventStore.ack(subscription, events)\nend\n```\n\nUnsubscribe from all streams:\n\n```elixir\nalias MyApp.EventStore\n\n:ok = EventStore.unsubscribe_from_all_streams(\"example_all_subscription\")\n```\n\n##","ref":"subscriptions.html#subscribe-to-all-events"},{"type":"extras","title":"Subscribe to single stream events - Subscriptions","doc":"Subscribe to events appended to a *single* stream:\n\n```elixir\nalias MyApp.EventStore\n\nstream_uuid = EventStore.UUID.uuid4()\n{:ok, subscription} = EventStore.subscribe_to_stream(stream_uuid, \"example_single_subscription\", self())\n\n# Wait for the subscription confirmation\nreceive do\n  {:subscribed, ^subscription} ->\n    IO.puts(\"Successfully subscribed to single stream\")\nend\n\nreceive do\n  {:events, events} ->\n    # Process events & acknowledge receipt\n    :ok = EventStore.ack(subscription, events)\nend\n```\n\nUnsubscribe from a single stream:\n\n```elixir\nalias MyApp.EventStore\n\n:ok = EventStore.unsubscribe_from_stream(stream_uuid, \"example_single_subscription\")\n```\n\n##","ref":"subscriptions.html#subscribe-to-single-stream-events"},{"type":"extras","title":"Start subscription from a given position - Subscriptions","doc":"You can choose to receive events from a given starting position.\n\nThe supported options are:\n\n  - `:origin` - Start receiving events from the beginning of the stream or all streams (default).\n  - `:current` - Subscribe to newly appended events only, skipping already persisted events.\n  - `event_number` (integer) - Specify an exact event number to subscribe from. This will be the same as the stream version for single stream subscriptions.\n\nExample all stream subscription that will receive new events appended after the subscription has been created:\n\n```elixir\nalias MyApp.EventStore\n\n{:ok, subscription} = EventStore.subscribe_to_all_streams(\"example_subscription\", self(), start_from: :current)\n```\n\n##","ref":"subscriptions.html#start-subscription-from-a-given-position"},{"type":"extras","title":"Event Filtering - Subscriptions","doc":"You can provide an event selector function that run in the subscription process, before sending the event to your mapper and subscriber. You can use this to filter events before dispatching to a subscriber.\n\nSubscribe to all streams and provide a `selector` function that only sends data that the selector function returns `true` for.\n\n```elixir\nalias EventStore.RecordedEvent\nalias MyApp.EventStore\n\nselector = fn %RecordedEvent{event_number: event_number} ->\n  rem(event_number) == 0\nend\n\n{:ok, subscription} = EventStore.subscribe_to_all_streams(\"example_subscription\", self(), selector: selector)\n\n# wait for the subscription confirmation\nreceive do\n  {:subscribed, ^subscription} ->\n    IO.puts(\"Successfully subscribed to all streams\")\nend\n\nreceive do\n  {:events, filtered_events} ->\n    # ... process events & ack receipt using last `event_number`\n    RecordedEvent{event_number: event_number} = List.last(filtered_events)\n\n    :ok = EventStore.ack(subscription, event_number)\nend\n```\n\n##","ref":"subscriptions.html#event-filtering"},{"type":"extras","title":"Mapping events - Subscriptions","doc":"You can provide an event mapping function that runs in the subscription process, before sending the event to your subscriber. You can use this to change the data received.\n\nSubscribe to all streams and provide a `mapper` function that sends only the event data:\n\n```elixir\nalias EventStore.RecordedEvent\nalias MyApp.EventStore\n\nmapper = fn %RecordedEvent{event_number: event_number, data: data} ->\n  {event_number, data}\nend\n\n{:ok, subscription} = EventStore.subscribe_to_all_streams(\"example_subscription\", self(), mapper: mapper)\n\n# wait for the subscription confirmation\nreceive do\n  {:subscribed, ^subscription} ->\n    IO.puts(\"Successfully subscribed to all streams\")\nend\n\nreceive do\n  {:events, mapped_events} ->\n    # ... process events & ack receipt using last `event_number`\n    {event_number, _data} = List.last(mapped_events)\n\n    :ok = EventStore.ack(subscription, event_number)\nend\n```\n\n#","ref":"subscriptions.html#mapping-events"},{"type":"extras","title":"Subscription concurrency - Subscriptions","doc":"A single persistent subscription can support multiple subscribers. Events will be distributed to subscribers evenly using a round-robin algorithm. The [competing consumers pattern](https://docs.microsoft.com/en-us/azure/architecture/patterns/competing-consumers) enables multiple subscribers to process events concurrently to optimise throughput, to improve scalability and availability, and to balance the workload.\n\nBy default a subscription will only allow a single subscriber but you can opt-in to concurrent subscriptions be providing a non-negative `concurrency_limit` as a subscription option.\n\n##","ref":"subscriptions.html#subscription-concurrency"},{"type":"extras","title":"Subscription concurrency configuration options - Subscriptions","doc":"- `concurrency_limit` defines the maximum number of concurrent subscribers allowed to connect to the subscription. By default only one subscriber may connect. If too many subscribers attempt to connect to the\n  subscription an `{:error, :too_many_subscribers}` is returned.\n\n- `buffer_size` limits how many in-flight events will be sent to the subscriber process before acknowledgement of successful processing. This limits the number of messages sent to the subscriber and stops their message queue from getting filled with events. Defaults to one in-flight event.\n\n- `partition_by` is an optional function used to partition events to subscribers. It can be used to guarantee processing order when multiple subscribers have subscribed to a single subscription as described in [Ordering guarantee](#ordering-guarantee) below. The function is passed a single argument (an `EventStore.RecordedEvent` struct) and must return the partition key. As an example to guarantee events for a single stream are processed serially, but different streams are processed concurrently, you could use the `stream_uuid` as the partition key.\n\n#","ref":"subscriptions.html#subscription-concurrency-configuration-options"},{"type":"extras","title":"Ordering guarantee - Subscriptions","doc":"With multiple subscriber processes connected to a single subscription the ordering of event processing is no longer guaranteed since events may be processed in differing amounts of time. This can cause problems if your event handling code expects events to be processed in the order they were originally appended to a steam.\n\nYou can use a `partition_by` function to guarantee ordering of events within a particular group (e.g. per stream) but still allow events for different groups to be processed concurrently.\n\n\nPartitioning gives you the benefits of competing consumers but still allows event ordering by partition where required.\n\n##","ref":"subscriptions.html#ordering-guarantee"},{"type":"extras","title":"Partition by example - Subscriptions","doc":"```elixir\nalias EventStore.RecordedEvent\nalias MyApp.EventStore\n\nby_stream = fn %RecordedEvent{stream_uuid: stream_uuid} -> stream_uuid end\n\n{:ok, _subscription} =\n  EventStore.subscribe_to_stream(stream_uuid, \"example\", self(),\n    concurrency_limit: 10,\n    partition_by: by_stream\n  )\n```\n\nThe above subscription would ensure that events for each stream are processed serially (by a single subscriber) in the order they were appended to the stream, but events for any other stream can be processed concurrently by another subscriber.\n\n#","ref":"subscriptions.html#partition-by-example"},{"type":"extras","title":"Example persistent subscriber - Subscriptions","doc":"Use a `GenServer` process to subscribe to the event store and track all notified events:\n\n```elixir\n# An example subscriber\ndefmodule Subscriber do\n  use GenServer\n\n  alias MyApp.EventStore\n\n  def start_link do\n    GenServer.start_link(__MODULE__, [])\n  end\n\n  def received_events(subscriber) do\n    GenServer.call(subscriber, :received_events)\n  end\n\n  def init(events) do\n    # Subscribe to events from all streams\n    {:ok, subscription} = EventStore.subscribe_to_all_streams(\"example_subscription\", self())\n\n    {:ok, %{events: events, subscription: subscription}}\n  end\n\n  # Successfully subscribed to all streams\n  def handle_info({:subscribed, subscription}, %{subscription: subscription} = state) do\n    {:noreply, state}\n  end\n\n  # Event notification\n  def handle_info({:events, events}, state) do\n    %{events: existing_events, subscription: subscription} = state\n\n    # Confirm receipt of received events\n    :ok = EventStore.ack(subscription, events)\n\n    {:noreply, %{state | events: existing_events ++ events}}\n  end\n\n  def handle_call(:received_events, _from, %{events: events} = state) do\n    {:reply, events, state}\n  end\nend\n```\n\nStart your subscriber process, which subscribes to all streams in the event store:\n\n```elixir\n{:ok, subscriber} = Subscriber.start_link()\n```\n\n#","ref":"subscriptions.html#example-persistent-subscriber"},{"type":"extras","title":"Deleting a persistent subscription - Subscriptions","doc":"You can delete a single stream or all stream subscription without requiring an active subscriber:\n\n```elixir\nalias MyApp.EventStore\n\n:ok = EventStore.delete_subscription(stream_uuid, subscription_name)\n:ok = EventStore.delete_all_streams_subscription(subscription_name)\n```\n\nDeleting the subscription will remove the subscription checkpoint allowing you to later create a subscription with the same name, using any start point.\n\nIf there is an active subscriber when deleting the subscription it will be stopped.","ref":"subscriptions.html#deleting-a-persistent-subscription"},{"type":"extras","title":"Event serialization","doc":"# Event serialization\n\nThe default serialization of event data and metadata uses Erlang's [external term format](http://erlang.org/doc/apps/erts/erl_ext_dist.html). This is not a recommended serialization format for production usage as backwards compatibility is only guaranteed going back at least two major releases.\n\nYou must implement the `EventStore.Serializer` behaviour to provide your preferred serialization format.","ref":"event-serialization.html"},{"type":"extras","title":"JSON serialization using Jason - Event serialization","doc":"EventStore includes a JSON serializer using Jason in the `EventStore.JsonSerializer` module. To include it, add `{:jason, \"~> 1.1\"}` to your application's mix dependencies and configure your EventStore as below.\n\n```elixir\nconfig :eventstore, MyApp.EventStore, serializer: EventStore.JsonSerializer\n```","ref":"event-serialization.html#json-serialization-using-jason"},{"type":"extras","title":"Example JSON serializer using Poison - Event serialization","doc":"The example serializer below serializes event data and metadata to JSON using the [Poison](https://github.com/devinus/poison) library.\n\n```elixir\ndefmodule JsonSerializer do\n  @moduledoc \"\"\"\n  A serializer that uses the JSON format.\n  \"\"\"\n\n  @behaviour EventStore.Serializer\n\n  @doc \"\"\"\n  Serialize given term to JSON binary data.\n  \"\"\"\n  def serialize(term) do\n    Poison.encode!(term)\n  end\n\n  @doc \"\"\"\n  Deserialize given JSON binary data to the expected type.\n  \"\"\"\n  def deserialize(binary, config) do\n    type = case Keyword.get(config, :type, nil) do\n      nil -> []\n      type -> type |> String.to_existing_atom |> struct\n    end\n    Poison.decode!(binary, as: type)\n  end\nend\n```\n\nConfigure the JSON serializer for your event store by setting the `serializer` option in the mix environment configuration file (e.g. `config/dev.exs`):\n\n```elixir\nconfig :eventstore, MyApp.EventStore, serializer: JsonSerializer\n```\n\nYou must set the `event_type` field to a string representing the type of event being persisted when using this serializer:\n\n```elixir\n%EventStore.EventData{\n  event_type: \"Elixir.ExampleEvent\",\n  data: %ExampleEvent{key: \"value\"},\n  metadata: %{user: \"someuser@example.com\"},\n}\n```\n\nYou can use `Atom.to_string/1` to get a string representation of a given event struct compatible with the example `JsonSerializer` module:\n\n```elixir\nevent = %ExampleEvent{key: \"value\"}\nevent_type = Atom.to_string(event.__struct__)  #=> \"Elixir.ExampleEvent\"\n```","ref":"event-serialization.html#example-json-serializer-using-poison"},{"type":"extras","title":"Running on a cluster of nodes","doc":"# Running on a cluster of nodes\n\nEventStore supports running on multiple nodes as either a [distributed Erlang](http://erlang.org/doc/reference_manual/distributed.html) cluster or as multiple single instance nodes.","ref":"cluster.html"},{"type":"extras","title":"Event publication - Running on a cluster of nodes","doc":"PostgreSQL's `LISTEN` / `NOTIFY` is used to pub/sub event notifications. A listener database connection process is started on each node. It connects to the database to listen for events and publishes them to interested subscription processes running on the node. The approach is the same regardless of whether distributed Erlang is used or not.","ref":"cluster.html#event-publication"},{"type":"extras","title":"Subscriptions - Running on a cluster of nodes","doc":"PostgreSQL's [advisory locks](https://www.postgresql.org/docs/current/static/explicit-locking.html#ADVISORY-LOCKS) are used to limit each uniquely named subscription to run at most once. This prevents multiple instances of a subscription from running on different nodes. Advisory locks are faster than table locks, are stored in memory to avoid table bloat, and are automatically cleaned up by the server at the end of the session.","ref":"cluster.html#subscriptions"},{"type":"extras","title":"Automatic cluster formation - Running on a cluster of nodes","doc":"You can use [libcluster](https://github.com/bitwalker/libcluster) to automatically form clusters of Erlang nodes, with either static or dynamic node membership.\n\nYou will need to include `libcluster` as an additional dependency in `mix.exs`:\n\n```elixir\ndefp deps do\n  [{:libcluster, \"~> 2.2\"}]\nend\n```\n\nThen configure your preferred cluster topology in the environment config (e.g. `config/config.exs`). An example is shown below using the standard Erlang `epmd` daemon strategy:\n\n```elixir\nconfig :libcluster,\n  topologies: [\n    example: [\n      strategy: Cluster.Strategy.Epmd,\n      config: [hosts: [:\"node1@127.0.0.1\", :\"node2@127.0.0.1\", :\"node3@127.0.0.1\"]],\n    ]\n  ]\n```\n\nPlease refer to the [`libcluster` documentation](https://hexdocs.pm/libcluster/) for more detail.\n\n#","ref":"cluster.html#automatic-cluster-formation"},{"type":"extras","title":"Starting a cluster - Running on a cluster of nodes","doc":"1. Run an [Erlang Port Mapper Daemon](http://erlang.org/doc/man/epmd.html) (epmd):\n\n      ```console\n      $ epmd -d\n      ```\n\n  2. Start an `iex` console per node:\n\n      ```console\n      $ MIX_ENV=distributed iex --name node1@127.0.0.1 -S mix\n      ```\n\n      ```console\n      $ MIX_ENV=distributed iex --name node2@127.0.0.1 -S mix\n      ```\n\n      ```console\n      $ MIX_ENV=distributed iex --name node3@127.0.0.1 -S mix\n      ```\n\nThe cluster will be automatically formed as soon as the nodes start.","ref":"cluster.html#starting-a-cluster"},{"type":"extras","title":"Static cluster topology and formation - Running on a cluster of nodes","doc":"Instead of using `libcluster` you can configure the `:kernel` application to wait for cluster formation before starting your application during node start up. This approach is useful when you have a static cluster topology that can be defined in config.\n\nThe `sync_nodes_optional` configuration specifies which nodes to attempt to connect to within the `sync_nodes_timeout` window, defined in milliseconds, before continuing with startup. There is also a `sync_nodes_mandatory` setting which can be used to enforce all nodes are connected within the timeout window or else the node terminates.\n\nEach node requires its own individual configuration, listing the other nodes in the cluster:\n\n```elixir\n# node1 config\nconfig :kernel,\n  sync_nodes_optional: [:\"node2@192.168.1.1\", :\"node3@192.168.1.2\"],\n  sync_nodes_timeout: 30_000\n```\n\nThe `sync_nodes_timeout` can be configured as `:infinity` to wait indefinitely for all nodes to\nconnect. All involved nodes must have the same value for `sync_nodes_timeout`.\n\nThe above approach will *only work* for Elixir releases. You will need to use [Erlang's `sys.config`](http://erlang.org/doc/man/config.html) file for development purposes.\n\nThe Erlang equivalent of the `:kernerl` mix config, as above, is:\n\n```erlang\n% node1.sys.config\n[{kernel,\n  [\n    {sync_nodes_optional, ['node2@127.0.0.1', 'node3@127.0.0.1']},\n    {sync_nodes_timeout, 30000}\n  ]}\n].\n```\n\n#","ref":"cluster.html#static-cluster-topology-and-formation"},{"type":"extras","title":"Starting a cluster - Running on a cluster of nodes","doc":"1. Run an [Erlang Port Mapper Daemon](http://erlang.org/doc/man/epmd.html) (epmd):\n\n      ```console\n      $ epmd -d\n      ```\n\n  2. Start an `iex` console per node:\n\n      ```console\n      $ MIX_ENV=distributed iex --name node1@127.0.0.1 --erl \"-config cluster/node1.sys.config\" -S mix\n      ```\n\n      ```console\n      $ MIX_ENV=distributed iex --name node2@127.0.0.1 --erl \"-config cluster/node2.sys.config\" -S mix\n      ```\n\n      ```console\n      $ MIX_ENV=distributed iex --name node3@127.0.0.1 --erl \"-config cluster/node3.sys.config\" -S mix\n      ```\n\nThe node specific ` .sys.config` files ensure the cluster is formed before starting your application, assuming this occurs within the 30 seconds timeout.\n\nOnce the cluster has formed, you can use your event store module from any node.","ref":"cluster.html#starting-a-cluster"},{"type":"extras","title":"Usage - Running on a cluster of nodes","doc":"Using the event store when run on a cluster of nodes is identical to single node usage. You can subscribe to a stream, or all streams, on one node and append events to the stream on another. The subscription will be notified of the appended events.\n\n#","ref":"cluster.html#usage"},{"type":"extras","title":"Append events to a stream - Running on a cluster of nodes","doc":"```elixir\nalias EventStore.EventData\nalias MyApp.EventStore\n\ndefmodule ExampleEvent do\n  defstruct [:key]\nend\n\nstream_uuid = EventStore.UUID.uuid4()\n\nevents = [\n  %EventData{\n    event_type: \"Elixir.ExampleEvent\",\n    data: %ExampleEvent{key: \"value\"},\n    metadata: %{user: \"someuser@example.com\"},\n  }\n]\n\n:ok = EventStore.append_to_stream(stream_uuid, 0, events)\n```\n\n#","ref":"cluster.html#append-events-to-a-stream"},{"type":"extras","title":"Read all events - Running on a cluster of nodes","doc":"```elixir\nalias MyApp.EventStore\n\nrecorded_events = EventStore.stream_all_forward() |> Enum.to_list()\n```\n\n#","ref":"cluster.html#read-all-events"},{"type":"extras","title":"Subscribe to all Streams - Running on a cluster of nodes","doc":"```elixir\nalias MyApp.EventStore\n\n{:ok, subscription} = EventStore.subscribe_to_all_streams(\"example-subscription\", self(), start_from: :origin)\n\nreceive do\n  {:subscribed, ^subscription} ->\n    IO.puts(\"Successfully subscribed to all streams\")\nend\n\nreceive do\n  {:events, events} ->\n    IO.puts(\"Received events: #{inspect(events)}\")\n\n    :ok = EventStore.ack(subscription, events)\nend\n```","ref":"cluster.html#subscribe-to-all-streams"},{"type":"extras","title":"Upgrading an EventStore","doc":"# Upgrading an EventStore\n\nThe [CHANGELOG](https://github.com/commanded/eventstore/blob/master/CHANGELOG.md) is used to indicate when a schema migration is required for a given version of the EventStore.","ref":"upgrades.html"},{"type":"extras","title":"Migrating a database with `mix` - Upgrading an EventStore","doc":"You can upgrade an existing EventStore database using the following mix task:\n\n```shell\nmix event_store.migrate\n```\n\nThe above command expects you to have configured the event store(s) for your application, as shown in the following example:\n\n```elixir\n# config/config.exs\nconfig :my_app, event_stores: [MyApp.EventStore]\n```\n\nOr use `mix event_store.migrate -e MyApp.EventStore` to specify an event store as an argument.\n\nRun this command each time you need to upgrade. The command is idempotent and will only run pending migrations if there are any and can be safely run multiple times. It can also be run concurrently as a database lock is used to ensure only one migration is run at at time.\n\nIt is *always* worth taking a full backup of the EventStore database before applying an upgrade.\n\nCreating an EventStore database, using the `mix event_store.create` task, will always use the latest database schema.","ref":"upgrades.html#migrating-a-database-with-mix"},{"type":"extras","title":"Migrating a database using an Elixir release - Upgrading an EventStore","doc":"If you're using an Elixir release built with [mix release](https://hexdocs.pm/mix/Mix.Tasks.Release.html) you won't have `mix` available and won't be able to run the above command to migrate the database.\n\nInstead you can use the [EventStore.Tasks.Migrate](https://github.com/commanded/eventstore/blob/master/lib/event_store/tasks/migrate.ex) task module and [running one-off commands](https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-one-off-commands-eval-and-rpc) supported by Mix release, using a helper module defined like this:\n\n```elixir\ndefmodule MyApp.ReleaseTasks do\n  def migrate_event_store do\n    {:ok, _} = Application.ensure_all_started(:postgrex)\n    {:ok, _} = Application.ensure_all_started(:ssl)\n\n    :ok = Application.load(:my_app)\n\n    config = MyApp.EventStore.config()\n\n    :ok = EventStore.Tasks.Migrate.exec(config, [])    \n  end\nend\n```\n\nThen run:\n\n```shell\nbin/RELEASE_NAME eval \"MyApp.ReleaseTasks.migrate_event_store()\"\n```","ref":"upgrades.html#migrating-a-database-using-an-elixir-release"},{"type":"extras","title":"Upgrade guide v0.17.x to 1.0","doc":"# Upgrade guide v0.17.x to v1.0\n\nIn v0.17.x and earlier EventStore was a singleton application with all functionality exposed by the `EventStore` module. To [support multiple event stores](https://github.com/commanded/eventstore/pull/168) in a single Elixir application you must now define your own event store module using the `EventStore` macro. You can define as many event stores as you like, each one is isolated and can be configured to use its own Postgres database.\n\n#","ref":"0.17-1.0.html"},{"type":"extras","title":"Getting started - Upgrade guide v0.17.x to 1.0","doc":"First, you must define your own event store module using the `EventStore` macro:\n\n```elixir\n# lib/my_app/event_store.ex\ndefmodule MyApp.EventStore do\n  use EventStore, otp_app: :my_app\n\n  # Optional `init/1` function to modify config at runtime.\n  def init(config) do\n    {:ok, config}\n  end\nend\n```\n\nYou can name your event store module however you like and optionally implement an `init/1` callback function.\n\nSecondly, configure the `MyApp.EventStore` module (in `config/config.exs` or each environment's config):\n\n```elixir\nconfig :my_app, MyApp.EventStore,\n  serializer: EventStore.JsonSerializer,\n  username: \"postgres\",\n  password: \"postgres\",\n  database: \"myapp_eventstore\",\n  hostname: \"localhost\",\n  pool_size: 10\n```\n\n**Note:** To use an EventStore with Commanded you should configure the event\nstore to use Commanded's JSON serializer which provides additional support for\nJSON decoding:\n\n```elixir\nconfig :my_app, MyApp.EventStore, serializer: Commanded.Serialization.JsonSerializer\n```\n\nFinally, the event store module must be included within your application's supervision tree, inside the `start/2` function:\n\n```elixir\n# lib/my_app/application.ex\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      MyApp.EventStore\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nOptionally, you can configure the event store modules in `config/config.exs` to allow running the event store mix tasks without providing the event store module as a command line argument:\n\n```elixir\nconfig :my_app, event_stores: [MyApp.EventStore]\n```\n\nThe above configuration allows you to run `mix event_store.init` instead of `mix event_store.init -e MyApp.EventStore` (as an example).\n\n#","ref":"0.17-1.0.html#getting-started"},{"type":"extras","title":"Usage - Upgrade guide v0.17.x to 1.0","doc":"Use your event store module exactly as you would have previously used the `EventStore` itself.\n\n```elixir\n:ok = MyApp.EventStore.append_to_stream(stream_uuid, expected_version, events)\n```\n\nFor ease of upgrading you can alias your own event store module as `EventStore` allowing you to use it without making any further code changes:\n\n```elixir\nalias MyApp.EventStore, as: EventStore\n\n:ok = EventStore.append_to_stream(stream_uuid, expected_version, events)\n```","ref":"0.17-1.0.html#usage"}],"content_type":"text/markdown"}